@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
@@ -8,26 +8,25 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
 import re
 
-from genshi.builder import tag
-
 from trac.config import get_configinfo
 from trac.core import *
 from trac.loader import get_plugin_info
 from trac.perm import IPermissionRequestor
+from trac.util.html import tag
 from trac.util.translation import _
 from trac.web.api import IRequestHandler
-from trac.web.chrome import Chrome, INavigationContributor
+from trac.web.chrome import Chrome, INavigationContributor, accesskey
 
 
 class AboutModule(Component):
@@ -45,7 +44,8 @@ class AboutModule(Component):
 
     def get_navigation_items(self, req):
         yield ('metanav', 'about',
-               tag.a(_("About Trac"), href=req.href.about(), accesskey=9))
+               tag.a(_("About Trac"), href=req.href.about(),
+                     accesskey=accesskey(req, 9)))
 
     # IPermissionRequestor methods
 
@@ -63,7 +63,7 @@ class AboutModule(Component):
 
         if 'CONFIG_VIEW' in req.perm('config', 'systeminfo'):
             # Collect system information
-            data['systeminfo'] = self.env.get_systeminfo()
+            data['system_info'] = self.env.system_info
             Chrome(self.env).add_jquery_ui(req)
 
         if 'CONFIG_VIEW' in req.perm('config', 'plugins'):
@@ -78,4 +78,4 @@ class AboutModule(Component):
             # Collect config information
             data['config'] = get_configinfo(self.env)
 
-        return 'about.html', data, None
+        return 'about.html', data
