@@ -1,45 +1,51 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from functools import partial
 import os
 import pkg_resources
 import re
 import shutil
+from functools import partial
 
-from genshi.builder import tag
-
+from trac import log
 from trac.admin.api import IAdminPanelProvider
 from trac.core import *
 from trac.loader import get_plugin_info
-from trac.log import LOG_LEVELS
-from trac.perm import PermissionSystem, IPermissionRequestor
+from trac.log import LOG_LEVELS, LOG_LEVEL_ALIASES, LOG_LEVEL_ALIASES_MAP
+from trac.perm import IPermissionRequestor, PermissionExistsError, \
+                      PermissionSystem
 from trac.util.datefmt import all_timezones, pytz
-from trac.util.text import exception_to_unicode, \
-                           unicode_to_base64, unicode_from_base64
+from trac.util.html import tag
+from trac.util.text import exception_to_unicode, unicode_from_base64, \
+                           unicode_to_base64
 from trac.util.translation import _, Locale, get_available_locales, \
                                   ngettext, tag_
 from trac.web.api import HTTPNotFound, IRequestHandler, \
                          is_valid_default_handler
-from trac.web.chrome import add_notice, add_stylesheet, \
-                            add_warning, Chrome, INavigationContributor, \
-                            ITemplateProvider
+from trac.web.chrome import Chrome, INavigationContributor, \
+                            ITemplateProvider, add_notice, add_stylesheet, \
+                            add_warning
 from trac.wiki.formatter import format_to_html
 
 
+_valid_log_levels = set()
+_valid_log_levels.update(log.LOG_LEVELS)
+_valid_log_levels.update(log.LOG_LEVEL_ALIASES)
+
+
 class AdminModule(Component):
     """Web administration interface provider and panel manager."""
 
@@ -75,21 +81,10 @@ class AdminModule(Component):
         if not panels:
             raise HTTPNotFound(_("No administration panels available"))
 
-        def _panel_order(p1, p2):
-            if p1[::2] == ('general', 'basics'):
-                return -1
-            elif p2[::2] == ('general', 'basics'):
-                return 1
-            elif p1[0] == 'general':
-                if p2[0] == 'general':
-                    return cmp(p1[1:], p2[1:])
-                return -1
-            elif p2[0] == 'general':
-                if p1[0] == 'general':
-                    return cmp(p1[1:], p2[1:])
-                return 1
-            return cmp(p1, p2)
-        panels.sort(_panel_order)
+        def _panel_order(panel):
+            items = panel[::2]
+            return items[0] != 'general', items != ('general', 'basics'), items
+        panels.sort(key=_panel_order)
 
         cat_id = req.args.get('cat_id') or panels[0][0]
         panel_id = req.args.get('panel_id')
@@ -105,8 +100,8 @@ class AdminModule(Component):
         if not provider:
             raise HTTPNotFound(_("Unknown administration panel"))
 
-        template, data = \
-            provider.render_admin_panel(req, cat_id, panel_id, path_info)
+        resp = provider.render_admin_panel(req, cat_id, panel_id, path_info)
+        template, data = resp[:2]
 
         data.update({
             'active_cat': cat_id, 'active_panel': panel_id,
@@ -118,7 +113,7 @@ class AdminModule(Component):
         })
 
         add_stylesheet(req, 'common/css/admin.css')
-        return template, data, None
+        return resp
 
     # ITemplateProvider methods
 
@@ -207,7 +202,8 @@ class BasicsAdminPanel(Component):
             default_date_format = req.args.get('default_date_format')
             if default_date_format != 'iso8601':
                 default_date_format = ''
-            self.config.set('trac', 'default_date_format', default_date_format)
+            self.config.set('trac', 'default_date_format',
+                            default_date_format)
 
             default_dateinfo_format = req.args.get('default_dateinfo_format')
             if default_dateinfo_format not in ('relative', 'absolute'):
@@ -281,12 +277,12 @@ class LoggingAdminPanel(Component):
                     _("Unknown log type %(type)s", type=new_type),
                     _("Invalid log type")
                 )
-            new_file = req.args.get('log_file', 'trac.log')
+            new_file = req.args.get('log_file', log_file)
             if not new_file:
                 raise TracError(_("You must specify a log file"),
                                 _("Missing field"))
-            new_level = req.args.get('log_level')
-            if new_level not in LOG_LEVELS:
+            new_level = req.args.get('log_level', log_level)
+            if new_level not in _valid_log_levels:
                 raise TracError(
                     _("Unknown log level %(level)s", level=new_level),
                     _("Invalid log level"))
@@ -314,31 +310,29 @@ class LoggingAdminPanel(Component):
                     changed = True
                     log_type = new_type
 
-                if log_type == 'none':
-                    self.config.remove('logging', 'log_level')
+                if new_level != log_level:
+                    self.config.set('logging', 'log_level', new_level)
                     changed = True
-                else:
-                    if new_level != log_level:
-                        self.config.set('logging', 'log_level', new_level)
-                        changed = True
-                        log_level = new_level
+                    log_level = new_level
 
-                if log_type == 'file':
-                    if new_file != log_file:
-                        self.config.set('logging', 'log_file', new_file)
-                        changed = True
-                        log_file = new_file
-                else:
-                    self.config.remove('logging', 'log_file')
+                if new_file != log_file:
+                    self.config.set('logging', 'log_file', new_file)
                     changed = True
+                    log_file = new_file
 
             if changed:
                 _save_config(self.config, req, self.log),
             req.redirect(req.href.admin(cat, page))
 
+        # Order log levels by priority value, with aliases excluded.
+        all_levels = sorted(log.LOG_LEVEL_MAP, key=log.LOG_LEVEL_MAP.get,
+                            reverse=True)
+        log_levels = [level for level in all_levels if level in log.LOG_LEVELS]
+        log_level = LOG_LEVEL_ALIASES_MAP.get(log_level, log_level)
+
         data = {
             'type': log_type, 'types': log_types,
-            'level': log_level, 'levels': LOG_LEVELS,
+            'level': log_level, 'levels': log_levels,
             'file': log_file, 'dir': log_dir
         }
         return 'admin_logging.html', {'log': data}
@@ -361,7 +355,6 @@ class PermissionAdminPanel(Component):
 
     def render_admin_panel(self, req, cat, page, path_info):
         perm = PermissionSystem(self.env)
-        all_permissions = perm.get_all_permissions()
         all_actions = perm.get_actions()
 
         if req.method == 'POST':
@@ -377,66 +370,55 @@ class PermissionAdminPanel(Component):
                                   "permission names."))
 
             # Grant permission to subject
-            if req.args.get('add') and subject and action:
+            if 'add' in req.args and subject and action:
                 req.perm('admin', 'general/perm').require('PERMISSION_GRANT')
                 if action not in all_actions:
                     raise TracError(_("Unknown action"))
                 req.perm.require(action)
-                if (subject, action) not in all_permissions:
+                try:
                     perm.grant_permission(subject, action)
+                except TracError as e:
+                    add_warning(req, e)
+                else:
                     add_notice(req, _("The subject %(subject)s has been "
                                       "granted the permission %(action)s.",
                                       subject=subject, action=action))
-                else:
-                    add_warning(req, _("The permission %(action)s was already "
-                                       "granted to %(subject)s.",
-                                       action=action, subject=subject))
 
             # Add subject to group
-            elif req.args.get('add') and subject and group:
+            elif 'add' in req.args and subject and group:
                 req.perm('admin', 'general/perm').require('PERMISSION_GRANT')
-                for action in perm.get_user_permissions(group):
-                    if not action in all_actions: # plugin disabled?
-                        self.env.log.warn("Adding %s to group %s: "
-                            "Permission %s unavailable, skipping perm check.",
-                            subject, group, action)
-                    else:
-                        req.perm.require(action,
-                            message=_("The subject %(subject)s was not added "
-                                      "to the group %(group)s because the "
-                                      "group has %(perm)s permission and "
-                                      "users cannot grant permissions they "
-                                      "don't possess.", subject=subject,
-                                      group=group, perm=action))
-                if (subject, group) not in all_permissions:
+                for action in sorted(
+                        perm.get_user_permissions(group, expand_meta=False)):
+                    req.perm.require(action,
+                        message=_("The subject %(subject)s was not added to "
+                                  "the group %(group)s because the group has "
+                                  "%(perm)s permission and users cannot grant "
+                                  "permissions they don't possess.",
+                                  subject=subject, group=group, perm=action))
+                try:
                     perm.grant_permission(subject, group)
-                    add_notice(req, _("The subject %(subject)s has been added "
-                                      "to the group %(group)s.",
-                                      subject=subject, group=group))
+                except TracError as e:
+                    add_warning(req, e)
                 else:
-                    add_warning(req, _("The subject %(subject)s was already "
-                                       "added to the group %(group)s.",
-                                       subject=subject, group=group))
+                    add_notice(req, _("The subject %(subject)s has been "
+                                      "added to the group %(group)s.",
+                                      subject=subject, group=group))
 
             # Copy permissions to subject
-            elif req.args.get('copy') and subject and target:
+            elif 'copy' in req.args and subject and target:
                 req.perm('admin', 'general/perm').require('PERMISSION_GRANT')
 
-                subject_permissions = [i[1] for i in all_permissions
-                                            if i[0] == subject and
-                                               i[1].isupper()]
+                subject_permissions = perm.get_users_dict().get(subject, [])
                 if not subject_permissions:
-                    add_warning(req,_("The subject %(subject)s does not "
-                                      "have any permissions.",
-                                      subject=subject))
+                    add_warning(req, _("The subject %(subject)s does not "
+                                       "have any permissions.",
+                                       subject=subject))
 
                 for action in subject_permissions:
-                    if (target, action) in all_permissions:
-                        continue
-                    if not action in all_actions: # plugin disabled?
-                        self.env.log.warn("Skipped granting %s to %s: "
-                                          "permission unavailable.",
-                                          action, target)
+                    if action not in all_actions:  # plugin disabled?
+                        self.log.warning("Skipped granting %s to %s: "
+                                         "permission unavailable.",
+                                         action, target)
                     else:
                         if action not in req.perm:
                             add_warning(req,
@@ -446,19 +428,21 @@ class PermissionAdminPanel(Component):
                                           "permissions they don't possess.",
                                           action=action, subject=subject))
                             continue
-                        perm.grant_permission(target, action)
-                        add_notice(req, _("The subject %(subject)s has "
-                                          "been granted the permission "
-                                          "%(action)s.",
-                                          subject=target, action=action))
+                        try:
+                            perm.grant_permission(target, action)
+                        except PermissionExistsError:
+                            pass
+                        else:
+                            add_notice(req, _("The subject %(subject)s has "
+                                              "been granted the permission "
+                                              "%(action)s.",
+                                              subject=target, action=action))
                 req.redirect(req.href.admin(cat, page))
 
             # Remove permissions action
-            elif req.args.get('remove') and req.args.get('sel'):
+            elif 'remove' in req.args and 'sel' in req.args:
                 req.perm('admin', 'general/perm').require('PERMISSION_REVOKE')
-                sel = req.args.get('sel')
-                sel = sel if isinstance(sel, list) else [sel]
-                for key in sel:
+                for key in req.args.getlist('sel'):
                     subject, action = key.split(':', 1)
                     subject = unicode_from_base64(subject)
                     action = unicode_from_base64(action)
@@ -471,6 +455,7 @@ class PermissionAdminPanel(Component):
 
         return 'admin_perms.html', {
             'actions': all_actions,
+            'allowed_actions': [a for a in all_actions if a in req.perm],
             'perms': perm.get_users_dict(),
             'groups': perm.get_groups_dict(),
             'unicode_to_base64': unicode_to_base64
@@ -497,7 +482,7 @@ class PluginAdminPanel(Component):
                 self._do_update(req)
             anchor = ''
             if 'plugin' in req.args:
-                anchor = '#no%d' % (int(req.args.get('plugin')) + 1)
+                anchor = '#no%d' % (req.args.getint('plugin') + 1)
             req.redirect(req.href.admin(cat, page) + anchor)
 
         return self._render_view(req)
@@ -532,11 +517,11 @@ class PluginAdminPanel(Component):
         except AttributeError:
             # OS_BINARY not available on every platform
             pass
-        with os.fdopen(os.open(target_path, flags, 0666), 'w') as target_file:
+        with os.fdopen(os.open(target_path, flags, 0o666), 'w') as target_file:
             shutil.copyfileobj(upload.file, target_file)
             self.log.info("Plugin %s installed to %s", plugin_filename,
                           target_path)
-        # TODO: Validate that the uploaded file is actually a valid Trac plugin
+        # TODO: Validate that the uploaded file is a valid Trac plugin
 
         # Make the environment reset itself on the next request
         self.env.config.touch()
@@ -556,7 +541,7 @@ class PluginAdminPanel(Component):
         self.env.config.touch()
 
     def _do_update(self, req):
-        """Update component enablement."""
+        """Update component enable state."""
         components = req.args.getlist('component')
         enabled = req.args.getlist('enable')
         added, removed = [], []
