@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009 Edgewall Software
+# Copyright (C) 2009-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import functools
 
@@ -138,8 +138,8 @@ def cached(fn_or_attr=None):
         class WikiSystem(Component):
             @cached
             def pages(self):
-                return set(name for name, in self.env.db_query(
-                               "SELECT DISTINCT name FROM wiki"))
+                return {name for name, in self.env.db_query(
+                            "SELECT DISTINCT name FROM wiki")}
 
     Otherwise, when the decorator is used in non-"singleton" objects,
     a string specifying the name of an attribute containing a string
