@@ -1,28 +1,27 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import copy
 import os.path
 import re
 from ConfigParser import ConfigParser, ParsingError
 
-from genshi.builder import tag
-
 from trac.admin import AdminCommandError, IAdminCommandProvider
 from trac.core import Component, ExtensionPoint, TracError, implements
 from trac.util import AtomicFile, as_bool
-from trac.util.compat import OrderedDict, wait_for_file_mtime_change
+from trac.util.compat import wait_for_file_mtime_change
+from trac.util.html import tag
 from trac.util.text import cleandoc, printout, to_unicode, to_utf8
 from trac.util.translation import N_, _, dgettext, tag_
 
@@ -91,8 +90,7 @@ class UnicodeConfigParser(ConfigParser):
 
     def __init__(self, ignorecase_option=True, **kwargs):
         self._ignorecase_option = ignorecase_option
-        dict_type = kwargs.pop('dict_type', None) or OrderedDict
-        ConfigParser.__init__(self, dict_type=dict_type, **kwargs)
+        ConfigParser.__init__(self, **kwargs)
 
     def optionxform(self, option):
         if self._ignorecase_option:
@@ -833,7 +831,7 @@ class ChoiceOption(Option):
                  doc_args=None, case_sensitive=True):
         Option.__init__(self, section, name, to_unicode(choices[0]), doc,
                         doc_domain, doc_args)
-        self.choices = list(set(to_unicode(c).strip() for c in choices))
+        self.choices = list({to_unicode(c).strip() for c in choices})
         self.case_sensitive = case_sensitive
 
     def accessor(self, section, name, default):
@@ -932,15 +930,13 @@ class OrderedExtensionsOption(ListOption):
                          for idx, impl in enumerate(not_found)),
                      option=tag.code("[%s] %s" % (self.section, self.name))))
 
-        def compare(x, y):
-            x, y = x.__class__.__name__, y.__class__.__name__
-            if x not in order:
-                return int(y in order)
-            if y not in order:
-                return -int(x in order)
-            return cmp(order.index(x), order.index(y))
-        components.sort(compare)
-        return components
+        def key(impl):
+            name = impl.__class__.__name__
+            if name in order:
+                return 0, order.index(name)
+            else:
+                return 1, components.index(impl)
+        return sorted(components, key=key)
 
 
 class ConfigurationAdmin(Component):
