@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Jeff Weiss <trac@jeffweiss.org>
 # Copyright (C) 2006 Andres Salomon <dilinger@athenacr.com>
@@ -8,26 +8,25 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import re
 import sys
-import types
-
-from genshi.core import Markup
+from contextlib import closing
 
+from trac.api import IEnvironmentSetupParticipant
 from trac.core import *
 from trac.config import Option
 from trac.db.api import ConnectionBase, DatabaseManager, IDatabaseConnector, \
                         get_column_names, parse_connection_uri
 from trac.db.util import ConnectionWrapper, IterableCursor
-from trac.env import IEnvironmentSetupParticipant, ISystemInfoProvider
 from trac.util import as_int, get_pkginfo
+from trac.util.html import Markup
 from trac.util.compat import close_fds
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
@@ -35,15 +34,14 @@ from trac.util.translation import _
 _like_escape_re = re.compile(r'([/_%])')
 
 try:
-    import MySQLdb
-    import MySQLdb.cursors
+    import pymysql
 except ImportError:
-    MySQLdb = None
-    mysqldb_version = None
+    pymysql = None
+    pymsql_version = None
 else:
-    mysqldb_version = get_pkginfo(MySQLdb).get('version', MySQLdb.__version__)
+    pymsql_version = get_pkginfo(pymysql).get('version', pymysql.__version__)
 
-    class MySQLUnicodeCursor(MySQLdb.cursors.Cursor):
+    class MySQLUnicodeCursor(pymysql.cursors.Cursor):
         def _convert_row(self, row):
             return tuple(v.decode('utf-8') if isinstance(v, str) else v
                          for v in row)
@@ -62,6 +60,11 @@ else:
             return [self._convert_row(row) for row in rows] \
                    if rows is not None else []
 
+    class MySQLSilentCursor(MySQLUnicodeCursor):
+        def _show_warnings(self, conn=None):
+            pass
+
+
 # Mapping from "abstract" SQL types to DB-specific types
 _type_map = {
     'int64': 'bigint',
@@ -69,6 +72,10 @@ _type_map = {
 }
 
 
+def _quote(identifier):
+    return "`%s`" % identifier.replace('`', '``')
+
+
 class MySQLConnector(Component):
     """Database connector for MySQL version 4.1 and greater.
 
@@ -86,8 +93,7 @@ class MySQLConnector(Component):
      * `read_default_group`: Configuration group to use from the default file
      * `unix_socket`: Use a Unix socket at the given path to connect
     """
-    implements(IDatabaseConnector, IEnvironmentSetupParticipant,
-               ISystemInfoProvider)
+    implements(IDatabaseConnector, IEnvironmentSetupParticipant)
 
     required = False
 
@@ -95,14 +101,12 @@ class MySQLConnector(Component):
         """Location of mysqldump for MySQL database backups""")
 
     def __init__(self):
-        self._mysql_version = None
-
-    # ISystemInfoProvider methods
-
-    def get_system_info(self):
-        if self.required:
-            yield 'MySQL', self._mysql_version
-            yield 'MySQLdb', mysqldb_version
+        if pymysql:
+            self._mysql_version = \
+                'server: (not-connected), client: "%s", thread-safe: %s' % \
+                (pymysql.get_client_info(), pymysql.thread_safe())
+        else:
+            self._mysql_version = None
 
     # IDatabaseConnector methods
 
@@ -115,13 +119,13 @@ class MySQLConnector(Component):
         if not self.required:
             self._mysql_version = \
                 'server: "%s", client: "%s", thread-safe: %s' \
-                % (cnx.cnx.get_server_info(), MySQLdb.get_client_info(),
-                   MySQLdb.thread_safe())
+                % (cnx.cnx.get_server_info(), pymysql.get_client_info(),
+                   pymysql.thread_safe())
             self.required = True
         return cnx
 
     def get_exceptions(self):
-        return MySQLdb
+        return pymysql
 
     def init_db(self, path, schema=None, log=None, user=None, password=None,
                 host=None, port=None, params={}):
@@ -156,9 +160,8 @@ class MySQLConnector(Component):
     def _max_bytes(self, cnx):
         if cnx is None:
             connector, args = DatabaseManager(self.env).get_connector()
-            cnx = connector.get_connection(**args)
-            charset = cnx.charset
-            cnx.close()
+            with closing(connector.get_connection(**args)) as cnx:
+                charset = cnx.charset
         else:
             charset = cnx.charset
         return 4 if charset == 'utf8mb4' else 3
@@ -177,7 +180,7 @@ class MySQLConnector(Component):
         limit = min(self._max_key_length / (max_bytes * len(columns)),
                     limit_col)
         for c in columns:
-            name = '`%s`' % c
+            name = _quote(c)
             table_col = filter((lambda x: x.name == c), table.columns)
             if len(table_col) == 1 and table_col[0].type.lower() == 'text':
                 if table_col[0].key_size is not None:
@@ -192,7 +195,7 @@ class MySQLConnector(Component):
     def to_sql(self, table, max_bytes=None):
         if max_bytes is None:
             max_bytes = self._max_bytes(None)
-        sql = ['CREATE TABLE %s (' % table.name]
+        sql = ['CREATE TABLE %s (' % _quote(table.name)]
         coldefs = []
         for column in table.columns:
             ctype = column.type
@@ -202,7 +205,7 @@ class MySQLConnector(Component):
                 # Override the column type, as a text field cannot
                 # use auto_increment.
                 column.type = 'int'
-            coldefs.append('    `%s` %s' % (column.name, ctype))
+            coldefs.append('    %s %s' % (_quote(column.name), ctype))
         if len(table.key) > 0:
             coldefs.append('    PRIMARY KEY (%s)' %
                            self._collist(table, table.key,
@@ -212,9 +215,10 @@ class MySQLConnector(Component):
 
         for index in table.indices:
             unique = 'UNIQUE' if index.unique else ''
-            yield 'CREATE %s INDEX %s_%s_idx ON %s (%s)' % (unique, table.name,
-                  '_'.join(index.columns), table.name,
-                  self._collist(table, index.columns, max_bytes=max_bytes))
+            idxname = '%s_%s_idx' % (table.name, '_'.join(index.columns))
+            yield 'CREATE %s INDEX %s ON %s (%s)' % \
+                  (unique, _quote(idxname), _quote(table.name),
+                   self._collist(table, index.columns, max_bytes=max_bytes))
 
     def alter_column_types(self, table, columns):
         """Yield SQL statements altering the type of one or more columns of
@@ -278,6 +282,10 @@ class MySQLConnector(Component):
             raise TracError(_("No destination file created"))
         return dest_file
 
+    def get_system_info(self):
+        yield 'MySQL', self._mysql_version
+        yield pymysql.__name__, pymsql_version
+
     # IEnvironmentSetupParticipant methods
 
     def environment_created(self):
@@ -331,7 +339,7 @@ class MySQLConnector(Component):
                        "'default_storage_engine','storage_engine',"
                        "'default_tmp_storage_engine',"
                        "'character_set_database','collation_database')")
-        vars = dict((row[0].lower(), row[1]) for row in cursor)
+        vars = {row[0].lower(): row[1] for row in cursor}
 
         engine = vars.get('default_storage_engine') or \
                  vars.get('storage_engine')
@@ -370,7 +378,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             password = ''
         if port is None:
             port = 3306
-        opts = {}
+        opts = {'charset': 'utf8'}
         for name, value in params.iteritems():
             key = name.encode('utf-8')
             if name == 'read_default_group':
@@ -381,22 +389,32 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
                 opts[key] = value.encode(sys.getfilesystemencoding())
             elif name in ('compress', 'named_pipe'):
                 opts[key] = as_int(value, 0)
+            elif name == 'charset':
+                value = value.lower()
+                if value in ('utf8', 'utf8mb4'):
+                    opts[key] = value
+                else:
+                    self.log.warning("Invalid connection string parameter "
+                                     "'%s=%s'", name, value)
             else:
                 self.log.warning("Invalid connection string parameter '%s'",
                                  name)
-        cnx = MySQLdb.connect(db=path, user=user, passwd=password, host=host,
-                              port=port, charset='utf8', **opts)
-        self.schema = path
-        if hasattr(cnx, 'encoders'):
-            # 'encoders' undocumented but present since 1.2.1 (r422)
-            cnx.encoders[Markup] = cnx.encoders[types.UnicodeType]
+        cnx = pymysql.connect(db=path, user=user, passwd=password, host=host,
+                              port=port, **opts)
         cursor = cnx.cursor()
         cursor.execute("SHOW VARIABLES WHERE "
                        " variable_name='character_set_database'")
         self.charset = cursor.fetchone()[1]
-        if self.charset != 'utf8':
-            cnx.query("SET NAMES %s" % self.charset)
-            cnx.store_result()
+        cursor.close()
+        if self.charset != opts['charset']:
+            cnx.close()
+            opts['charset'] = self.charset
+            cnx = pymysql.connect(db=path, user=user, passwd=password,
+                                  host=host, port=port, **opts)
+        self.schema = path
+        if hasattr(cnx, 'encoders'):
+            # 'encoders' undocumented but present since 1.2.1 (r422)
+            cnx.encoders[Markup] = cnx.encoders[unicode]
         ConnectionWrapper.__init__(self, cnx, log)
         self._is_closed = False
 
@@ -407,14 +425,14 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         self.cnx.ping()
         try:
             self.cnx.rollback()
-        except MySQLdb.ProgrammingError:
+        except pymysql.ProgrammingError:
             self._is_closed = True
 
     def close(self):
         if not self._is_closed:
             try:
                 self.cnx.close()
-            except MySQLdb.ProgrammingError:
+            except pymysql.ProgrammingError:
                 pass # this error would mean it's already closed.  So, ignore
             self._is_closed = True
 
@@ -429,7 +447,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         return 'concat(%s)' % ', '.join(args)
 
     def drop_column(self, table, column):
-        cursor = MySQLdb.cursors.Cursor(self.cnx)
+        cursor = pymysql.cursors.Cursor(self.cnx)
         if column in self.get_column_names(table):
             quoted_table = self.quote(table)
             cursor.execute("SHOW INDEX FROM %s" % quoted_table)
@@ -451,8 +469,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
                            (quoted_table, self.quote(column)))
 
     def drop_table(self, table):
-        cursor = MySQLdb.cursors.Cursor(self.cnx)
-        cursor._defer_warnings = True  # ignore "Warning: Unknown table ..."
+        cursor = MySQLSilentCursor(self.cnx)
         cursor.execute("DROP TABLE IF EXISTS " + self.quote(table))
 
     def get_column_names(self, table):
@@ -466,6 +483,9 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
     def get_last_id(self, cursor, table, column='id'):
         return cursor.lastrowid
 
+    def get_sequence_names(self):
+        return []
+
     def get_table_names(self):
         rows = self.execute("""
             SELECT table_name FROM information_schema.tables
@@ -473,6 +493,13 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             """, (self.schema,))
         return [row[0] for row in rows]
 
+    def has_table(self, table):
+        rows = self.execute("""
+            SELECT EXISTS (SELECT * FROM information_schema.columns
+                           WHERE table_schema=%s AND table_name=%s)
+            """, (self.schema, table))
+        return bool(rows[0][0])
+
     def like(self):
         return "LIKE %%s COLLATE %s_general_ci ESCAPE '/'" % self.charset
 
@@ -485,19 +512,25 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             return table_names
         cursor = self.cursor()
         cursor.execute("""
-            SELECT table_name, auto_increment
-            FROM information_schema.tables
-            WHERE table_schema=%s""", (self.schema,))
-        for table, auto_increment in cursor.fetchall():
+            SELECT t.table_name,
+                   EXISTS (SELECT * FROM information_schema.columns AS c
+                           WHERE c.table_schema=t.table_schema
+                           AND c.table_name=t.table_name
+                           AND extra='auto_increment')
+            FROM information_schema.tables AS t
+            WHERE t.table_schema=%s
+            """, (self.schema,))
+        for table, has_autoinc in cursor.fetchall():
             table_names.append(table)
-            if auto_increment is None or auto_increment == 1:
+            quoted = self.quote(table)
+            if not has_autoinc:
                 # DELETE FROM is preferred to TRUNCATE TABLE, as the
-                # auto_increment is not used or it is 1.
-                cursor.execute("DELETE FROM %s" % table)
+                # auto_increment is not used.
+                cursor.execute("DELETE FROM %s" % quoted)
             else:
                 # TRUNCATE TABLE is preferred to DELETE FROM, as we
                 # need to reset the auto_increment in MySQL.
-                cursor.execute("TRUNCATE TABLE %s" % table)
+                cursor.execute("TRUNCATE TABLE %s" % quoted)
         return table_names
 
     def prefix_match(self):
@@ -507,7 +540,8 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         return self.like_escape(prefix) + '%'
 
     def quote(self, identifier):
-        return "`%s`" % identifier.replace('`', '``')
+        """Return the quoted identifier."""
+        return _quote(identifier)
 
     def update_sequence(self, cursor, table, column='id'):
         # MySQL handles sequence updates automagically
