@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -18,16 +18,15 @@ import errno
 import os
 import re
 import weakref
-
-from genshi.builder import tag
+from contextlib import closing
 
 from trac.config import ConfigurationError, ListOption
 from trac.core import Component, TracError, implements
 from trac.db.api import ConnectionBase, IDatabaseConnector
 from trac.db.schema import Table, Column, Index
 from trac.db.util import ConnectionWrapper, IterableCursor
-from trac.env import ISystemInfoProvider
 from trac.util import get_pkginfo, getuser, lazy
+from trac.util.html import tag
 from trac.util.translation import _, tag_
 
 _like_escape_re = re.compile(r'([/_%])')
@@ -46,7 +45,7 @@ pysqlite_version_string = get_pkginfo(sqlite).get('version',
                                                   '%d.%d.%s'
                                                   % pysqlite_version)
 min_sqlite_version = (3, 0, 0)
-min_pysqlite_version = (2, 4, 1)  # version provided by Python 2.6
+min_pysqlite_version = (2, 6, 0)  # version provided by Python 2.7
 
 
 class PyFormatCursor(sqlite.Cursor):
@@ -73,7 +72,7 @@ class PyFormatCursor(sqlite.Cursor):
 
 # EagerCursor taken from the example in pysqlite's repository:
 #
-#   http://code.google.com/p/pysqlite/source/browse/misc/eager.py
+#   https://github.com/ghaering/pysqlite/blob/master/misc/eager.py
 #
 # Only change is to subclass it from PyFormatCursor instead of
 # sqlite.Cursor.
@@ -149,14 +148,13 @@ class SQLiteConnector(Component):
     sqlite:path/to/trac.db
     }}}
     """
-    implements(IDatabaseConnector, ISystemInfoProvider)
+    implements(IDatabaseConnector)
 
     required = False
 
     extensions = ListOption('sqlite', 'extensions',
         doc="""Paths to [https://sqlite.org/loadext.html sqlite extensions].
         The paths may be absolute or relative to the Trac environment.
-        (''since 0.12'')
         """)
 
     memory_cnx = None
@@ -164,13 +162,6 @@ class SQLiteConnector(Component):
     def __init__(self):
         self.error = None
 
-    # ISystemInfoProvider methods
-
-    def get_system_info(self):
-        if self.required:
-            yield 'SQLite', sqlite_version_string
-            yield 'pysqlite', pysqlite_version_string
-
     # IDatabaseConnector methods
 
     def get_supported_schemes(self):
@@ -182,9 +173,6 @@ class SQLiteConnector(Component):
         elif pysqlite_version < min_pysqlite_version:
             self.error = _("Need at least PySqlite %(version)s or higher",
                            version='%d.%d.%d' % min_pysqlite_version)
-        elif (2, 5, 2) <= pysqlite_version < (2, 5, 5):
-            self.error = _("PySqlite 2.5.2 - 2.5.4 break Trac, please use "
-                           "2.5.5 or higher")
         yield 'sqlite', -1 if self.error else 1
 
     def get_connection(self, path, log=None, params={}):
@@ -204,6 +192,14 @@ class SQLiteConnector(Component):
         return sqlite
 
     def init_db(self, path, schema=None, log=None, params={}):
+
+        def insert_schema(cursor, schema):
+            if schema is None:
+                from trac.db_default import schema
+            for table in schema:
+                for stmt in self.to_sql(table):
+                    cursor.execute(stmt)
+
         if path != ':memory:':
             # make the directory to hold the database
             if self.db_exists(path):
@@ -217,19 +213,15 @@ class SQLiteConnector(Component):
             # this direct connect will create the database if needed
             cnx = sqlite.connect(path, isolation_level=None,
                                  timeout=int(params.get('timeout', 10000)))
-            cursor = cnx.cursor()
-            _set_journal_mode(cursor, params.get('journal_mode'))
-            _set_synchronous(cursor, params.get('synchronous'))
+            with closing(cnx.cursor()) as cursor:
+                _set_journal_mode(cursor, params.get('journal_mode'))
+                set_synchronous(cursor, params.get('synchronous'))
+                insert_schema(cursor, schema)
             cnx.isolation_level = 'DEFERRED'
         else:
             cnx = self.get_connection(path, log, params)
-            cursor = cnx.cursor()
-        if schema is None:
-            from trac.db_default import schema
-        for table in schema:
-            for stmt in self.to_sql(table):
-                cursor.execute(stmt)
-        cursor.close()
+            with closing(cnx.cursor()) as cursor:
+                insert_schema(cursor, schema)
         cnx.commit()
 
     def destroy_db(self, path, log=None, params={}):
@@ -278,6 +270,10 @@ class SQLiteConnector(Component):
             raise TracError(_("No destination file created"))
         return dest_file
 
+    def get_system_info(self):
+        yield 'SQLite', sqlite_version_string
+        yield 'pysqlite', pysqlite_version_string
+
     @lazy
     def _extensions(self):
         _extensions = []
@@ -293,7 +289,7 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
 
     __slots__ = ['_active_cursors', '_eager']
 
-    poolable = sqlite_version >= (3, 3, 8) and pysqlite_version >= (2, 5, 0)
+    poolable = sqlite_version >= (3, 3, 8)
 
     def __init__(self, path, log=None, params={}):
         self.cnx = None
@@ -329,10 +325,9 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
                 cnx.load_extension(ext)
             cnx.enable_load_extension(False)
 
-        cursor = cnx.cursor()
-        _set_journal_mode(cursor, params.get('journal_mode'))
-        _set_synchronous(cursor, params.get('synchronous'))
-        cursor.close()
+        with closing(cnx.cursor()) as cursor:
+            _set_journal_mode(cursor, params.get('journal_mode'))
+            set_synchronous(cursor, params.get('synchronous'))
         cnx.isolation_level = 'DEFERRED'
         ConnectionWrapper.__init__(self, cnx, log)
 
@@ -343,7 +338,7 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
         return IterableCursor(cursor, self.log)
 
     def rollback(self):
-        for cursor in self._active_cursors.keys():
+        for cursor in self._active_cursors:
             cursor.close()
         self.cnx.rollback()
 
@@ -395,19 +390,23 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
             cursor.execute("DROP TABLE IF EXISTS " + self.quote(table))
 
     def get_column_names(self, table):
-        cursor = self.cursor()
-        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
-        return [row[1] for row in cursor]
+        return [row[1] for row in self._get_table_info(table)]
 
     def get_last_id(self, cursor, table, column='id'):
         return cursor.lastrowid
 
+    def get_sequence_names(self):
+        return []
+
     def get_table_names(self):
         rows = self.execute("""
             SELECT name FROM sqlite_master WHERE type='table'
             """)
         return [row[0] for row in rows]
 
+    def has_table(self, table):
+        return bool(self._get_table_info(table))
+
     def like(self):
         if sqlite_version >= (3, 1, 0):
             return "LIKE %s ESCAPE '/'"
@@ -437,16 +436,19 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
         return table_names
 
     def update_sequence(self, cursor, table, column='id'):
-        # SQLite handles sequence updates automagically
-        # http://www.sqlite.org/autoinc.html
+        # SQLite handles sequence updates automatically
+        # https://www.sqlite.org/autoinc.html
         pass
 
+    def _get_table_info(self, table):
+        cursor = self.cursor()
+        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
+        return list(cursor)
+
     def _get_table_schema(self, table):
         key = None
         items = []
-        cursor = self.cursor()
-        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
-        for row in cursor:
+        for row in self._get_table_info(table):
             column = row[1]
             type_ = row[2]
             pk = row[5]
@@ -457,6 +459,7 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
                 auto_increment = False
             items.append(Column(column, type=type_,
                                 auto_increment=auto_increment))
+        cursor = self.cursor()
         cursor.execute("PRAGMA index_list(%s)" % self.quote(table))
         for row in cursor.fetchall():
             index = row[1]
@@ -492,7 +495,7 @@ def _set_journal_mode(cursor, value):
                           value=value, version=sqlite_version_string))
 
 
-def _set_synchronous(cursor, value):
+def set_synchronous(cursor, value):
     if not value:
         return
     if value.isdigit():
