@@ -1,21 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 import re
+from contextlib import closing
 
 _sql_escape_percent_re = re.compile("""
     '(?:[^']+|'')*' |
@@ -124,10 +125,9 @@ class ConnectionWrapper(object):
         When more control is needed, use `cursor()`.
         """
         dql = self.check_select(query)
-        cursor = self.cnx.cursor()
-        cursor.execute(query, params if params is not None else [])
-        rows = cursor.fetchall() if dql else None
-        cursor.close()
+        with closing(self.cnx.cursor()) as cursor:
+            cursor.execute(query, params if params is not None else [])
+            rows = cursor.fetchall() if dql else None
         return rows
 
     __call__ = execute
@@ -142,10 +142,9 @@ class ConnectionWrapper(object):
         When more control is needed, use `cursor()`.
         """
         dql = self.check_select(query)
-        cursor = self.cnx.cursor()
-        cursor.executemany(query, params)
-        rows = cursor.fetchall() if dql else None
-        cursor.close()
+        with closing(self.cnx.cursor()) as cursor:
+            cursor.executemany(query, params)
+            rows = cursor.fetchall() if dql else None
         return rows
 
     def check_select(self, query):
