@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2014 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006-2007 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -59,39 +59,37 @@ that can be `read()`.
   corresponding ticket (#3332 as well).
 """
 
+import io
+import mimetypes
 import re
-from StringIO import StringIO
-
-from genshi import Markup, Stream
-from genshi.core import TEXT, START, END, START_NS, END_NS
-from genshi.builder import Fragment, tag
-from genshi.input import HTMLParser
+from collections import namedtuple
 
 from trac.config import IntOption, ListOption, Option
 from trac.core import Component, ExtensionPoint, Interface, TracError, \
                       implements
 from trac.resource import Resource
 from trac.util import Ranges, content_disposition
-from trac.util.text import exception_to_unicode, to_utf8, to_unicode
+from trac.util.html import Fragment, Markup, tag
+from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
 
 
-__all__ = ['Context', 'Mimeview', 'RenderingContext', 'get_mimetype',
+__all__ = ['Mimeview', 'RenderingContext', 'get_mimetype',
            'is_binary', 'detect_unicode', 'content_to_unicode', 'ct_mimetype']
 
 
 class RenderingContext(object):
-    """
-    A rendering context specifies ''how'' the content should be rendered.
+    """A rendering context specifies ''how'' the content should be rendered.
 
     It holds together all the needed contextual information that will be
     needed by individual renderer components.
 
-    To that end, a context keeps track of the Href instance (`.href`) which
-    should be used as a base for building URLs.
+    To that end, a context keeps track of the Href instance
+    (``.href``) which should be used as a base for building URLs.
 
-    It also provides a `PermissionCache`  (`.perm`) which can be used to
-    restrict the output so that only the authorized information is shown.
+    It also provides a `PermissionCache` (``.perm``) which can be used
+    to restrict the output so that only the authorized information is
+    shown.
 
     A rendering context may also be associated to some Trac resource which
     will be used as the implicit reference when rendering relative links
@@ -100,7 +98,7 @@ class RenderingContext(object):
 
     Rendering contexts can be nested, and a new context can be created from
     an existing context using the call syntax. The previous context can be
-    retrieved using the `.parent` attribute.
+    retrieved using the ``.parent`` attribute.
 
     For example, when rendering a wiki text of a wiki page, the context will
     be associated to a resource identifying that wiki page.
@@ -110,6 +108,7 @@ class RenderingContext(object):
     rendering the ticket descriptions.
 
     :since: version 1.0
+
     """
 
     def __init__(self, resource, href=None, perm=None):
@@ -131,14 +130,6 @@ class RenderingContext(object):
         self.perm = perm(resource) if perm and resource else perm
         self._hints = None
 
-    @staticmethod
-    def from_request(*args, **kwargs):
-        """:deprecated: since 1.0, use `web_context` instead. Will be removed
-                        in release 1.3.1.
-        """
-        from trac.web.chrome import web_context
-        return web_context(*args, **kwargs)
-
     def __repr__(self):
         path = []
         context = self
@@ -281,14 +272,6 @@ class RenderingContext(object):
             p = p.parent
         return p and p._hints
 
-
-class Context(RenderingContext):
-    """
-    :deprecated: since 1.0, use `RenderingContext` instead. `Context` is
-                 kept for compatibility and will be removed release 1.3.1.
-    """
-
-
 # Some common MIME types and their associated keywords and/or file extensions
 
 KNOWN_MIME_TYPES = {
@@ -349,7 +332,7 @@ KNOWN_MIME_TYPES = {
     'text/x-verilog':          'v',
     'text/x-vhdl':             'vhd',
 }
-for t in KNOWN_MIME_TYPES.keys():
+for t in KNOWN_MIME_TYPES:
     types = KNOWN_MIME_TYPES[t].split()
     if t.startswith('text/x-'):
         types.append(t[len('text/x-'):])
@@ -401,7 +384,6 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
     else:
         mimetype = None
         try:
-            import mimetypes
             # 2) mimetype from the suffix, using the `mimetypes` module
             mimetype = mimetypes.guess_type(filename)[0]
         except Exception:
@@ -525,7 +507,8 @@ class IHTMLPreviewRenderer(Interface):
         """
 
     def render(context, mimetype, content, filename=None, url=None):
-        """Render an XHTML preview of the raw `content` in a RenderingContext.
+        """Render an XHTML preview of the raw ``content`` in a
+        `RenderingContext`.
 
         The `content` might be:
          * a `str` object
@@ -542,6 +525,7 @@ class IHTMLPreviewRenderer(Interface):
         Can return the generated XHTML text as a single string or as an
         iterable that yields strings. In the latter case, the list will
         be considered to correspond to lines of text in the original content.
+
         """
 
 
@@ -613,7 +597,7 @@ class Content(object):
         if size == 0:
             return ''
         if self.content is None:
-            self.content = StringIO(self.input.read(self.max_size))
+            self.content = io.BytesIO(self.input.read(self.max_size))
         return self.content.read(size)
 
     def reset(self):
@@ -634,21 +618,21 @@ class Mimeview(Component):
         """Charset to be used when in doubt.""")
 
     tab_width = IntOption('mimeviewer', 'tab_width', 8,
-        """Displayed tab width in file preview. (''since 0.9'')""")
+        """Displayed tab width in file preview.""")
 
     max_preview_size = IntOption('mimeviewer', 'max_preview_size', 262144,
-        """Maximum file size for HTML preview. (''since 0.9'')""")
+        """Maximum file size for HTML preview.""")
 
     mime_map = ListOption('mimeviewer', 'mime_map',
         'text/x-dylan:dylan, text/x-idl:ice, text/x-ada:ads:adb',
         doc="""List of additional MIME types and keyword mappings.
         Mappings are comma-separated, and for each MIME type,
         there's a colon (":") separated list of associated keywords
-        or file extensions. (''since 0.10'')
+        or file extensions.
         """)
 
     mime_map_patterns = ListOption('mimeviewer', 'mime_map_patterns',
-        'text/plain:README|INSTALL|COPYING.*',
+        'text/plain:README(?!\.rst)|INSTALL(?!\.rst)|COPYING.*',
         doc="""List of additional MIME types associated to filename patterns.
         Mappings are comma-separated, and each mapping consists of a MIME type
         and a Python regexp used for matching filenames, separated by a colon
@@ -659,7 +643,8 @@ class Mimeview(Component):
         'application/octet-stream, application/pdf, application/postscript, '
         'application/msword, application/rtf',
         doc="""Comma-separated list of MIME types that should be treated as
-        binary data. (''since 0.11.5'')""")
+        binary data.
+        """)
 
     def __init__(self):
         self._mime_map = None
@@ -668,16 +653,21 @@ class Mimeview(Component):
     # Public API
 
     def get_supported_conversions(self, mimetype):
-        """Return a list of target MIME types in same form as
-        `IContentConverter.get_supported_conversions()`, but with the converter
-        component appended. Output is ordered from best to worst quality."""
+        """Return a list of target MIME types as instances of the `namedtuple`
+        `MimeConversion`. Output is ordered from best to worst quality.
+
+        The `MimeConversion` `namedtuple` has fields: key, name, extension,
+        in_mimetype, out_mimetype, quality, converter.
+        """
+        fields = ('key', 'name', 'extension', 'in_mimetype',
+                  'out_mimetype', 'quality', 'converter')
+        _MimeConversion = namedtuple('MimeConversion', fields)
         converters = []
-        for converter in self.converters:
-            conversions = converter.get_supported_conversions() or []
-            for k, n, e, im, om, q in conversions:
+        for c in self.converters:
+            for k, n, e, im, om, q in c.get_supported_conversions() or []:
                 if im == mimetype and q > 0:
-                    converters.append((k, n, e, im, om, q, converter))
-        converters = sorted(converters, key=lambda i: i[-2], reverse=True)
+                    converters.append(_MimeConversion(k, n, e, im, om, q, c))
+        converters = sorted(converters, key=lambda i: i.quality, reverse=True)
         return converters
 
     def convert_content(self, req, mimetype, content, key, filename=None,
@@ -700,17 +690,18 @@ class Mimeview(Component):
             mimetype = full_mimetype = 'text/plain'  # fallback if not binary
 
         # Choose best converter
-        candidates = list(self.get_supported_conversions(mimetype) or [])
-        candidates = [c for c in candidates if key in (c[0], c[4])]
+        candidates = [c for c in self.get_supported_conversions(mimetype)
+                        if key in (c.key, c.out_mimetype)]
         if not candidates:
             raise TracError(
                 _("No available MIME conversions from %(old)s to %(new)s",
                   old=mimetype, new=key))
 
         # First successful conversion wins
-        for ck, name, ext, input_mimettype, output_mimetype, quality, \
-                converter in candidates:
-            output = converter.convert_content(req, mimetype, content, ck)
+        for conversion in candidates:
+            output = conversion.converter.convert_content(req, mimetype,
+                                                          content,
+                                                          conversion.key)
             if output:
                 content, content_type = output
                 if iterable:
@@ -719,7 +710,7 @@ class Mimeview(Component):
                 else:
                     if not isinstance(content, basestring):
                         content = ''.join(content)
-                return content, content_type, ext
+                return content, content_type, conversion.extension
         raise TracError(
             _("No available MIME conversions from %(old)s to %(new)s",
               old=mimetype, new=key))
@@ -749,8 +740,6 @@ class Mimeview(Component):
         """
         if not content:
             return ''
-        if not isinstance(context, RenderingContext):
-            raise TypeError("RenderingContext expected (since 0.11)")
 
         # Ensure we have a MIME type for this content
         full_mimetype = mimetype
@@ -769,7 +758,7 @@ class Mimeview(Component):
             qr = renderer.get_quality_ratio(mimetype)
             if qr > 0:
                 candidates.append((qr, renderer))
-        candidates.sort(lambda x, y: cmp(y[0], x[0]))
+        candidates.sort(key=lambda item: -item[0])
 
         # Wrap file-like object so that it can be read multiple times
         if hasattr(content, 'read'):
@@ -783,49 +772,23 @@ class Mimeview(Component):
                 continue  # skip non-source renderers in force_source mode
             if isinstance(content, Content):
                 content.reset()
+            ann_names = ', '.join(annotations) if annotations else \
+                       'no annotations'
+            self.log.debug('Trying to render HTML preview using %s [%s]',
+                           renderer.__class__.__name__, ann_names)
+
+            # check if we need to perform a tab expansion
+            rendered_content = content
+            if getattr(renderer, 'expand_tabs', False):
+                if expanded_content is None:
+                    content = content_to_unicode(self.env, content,
+                                                 full_mimetype)
+                    expanded_content = content.expandtabs(self.tab_width)
+                rendered_content = expanded_content
             try:
-                ann_names = ', '.join(annotations) if annotations else \
-                           'no annotations'
-                self.log.debug('Trying to render HTML preview using %s [%s]',
-                               renderer.__class__.__name__, ann_names)
-
-                # check if we need to perform a tab expansion
-                rendered_content = content
-                if getattr(renderer, 'expand_tabs', False):
-                    if expanded_content is None:
-                        content = content_to_unicode(self.env, content,
-                                                     full_mimetype)
-                        expanded_content = content.expandtabs(self.tab_width)
-                    rendered_content = expanded_content
-
                 result = renderer.render(context, full_mimetype,
                                          rendered_content, filename, url)
-                if not result:
-                    continue
-
-                if not (force_source or getattr(renderer, 'returns_source',
-                                                False)):
-                    # Direct rendering of content
-                    if isinstance(result, basestring):
-                        if not isinstance(result, unicode):
-                            result = to_unicode(result)
-                        return Markup(to_unicode(result))
-                    elif isinstance(result, Fragment):
-                        return result.generate()
-                    else:
-                        return result
-
-                # Render content as source code
-                if annotations:
-                    m = context.req.args.get('marks') if context.req else None
-                    return self._render_source(context, result, annotations,
-                                               m and Ranges(m))
-                else:
-                    if isinstance(result, list):
-                        result = Markup('\n').join(result)
-                    return tag.div(class_='code')(tag.pre(result)).generate()
-
-            except Exception, e:
+            except Exception as e:
                 self.log.warning('HTML preview using %s with %r failed: %s',
                                  renderer.__class__.__name__, context,
                                  exception_to_unicode(e, traceback=True))
@@ -835,8 +798,33 @@ class Mimeview(Component):
                         _("HTML preview using %(renderer)s failed (%(err)s)",
                           renderer=renderer.__class__.__name__,
                           err=exception_to_unicode(e)))
+            else:
+                if not result:
+                    continue
 
-    def _render_source(self, context, stream, annotations, marks=None):
+                if not (force_source or getattr(renderer, 'returns_source',
+                                                False)):
+                    # Direct rendering of content
+                    if isinstance(result, basestring):
+                        return Markup(to_unicode(result))
+                    elif isinstance(result, Fragment):
+                        return result.__html__()
+                    else:
+                        return result
+
+                # Render content as source code
+                if annotations:
+                    marks = context.req.args.get('marks') if context.req \
+                            else None
+                    if marks:
+                        context.set_hints(marks=marks)
+                    return self._render_source(context, result, annotations)
+                else:
+                    if isinstance(result, list):
+                        result = Markup('\n').join(result)
+                    return tag.div(class_='code')(tag.pre(result))
+
+    def _render_source(self, context, lines, annotations):
         from trac.web.chrome import add_warning
         annotators, labels, titles = {}, {}, {}
         for annotator in self.annotators:
@@ -847,21 +835,17 @@ class Mimeview(Component):
                 annotators[atype] = annotator
         annotations = [a for a in annotations if a in annotators]
 
-        if isinstance(stream, list):
-            stream = HTMLParser(StringIO(u'\n'.join(stream)))
-        elif isinstance(stream, unicode):
-            text = stream
-            def linesplitter():
-                for line in text.splitlines(True):
-                    yield TEXT, line, (None, -1, -1)
-            stream = linesplitter()
+        if isinstance(lines, unicode):
+            lines = lines.splitlines(True)
+        # elif isinstance(lines, list):
+        #    pass # assume these are lines already
 
         annotator_datas = []
         for a in annotations:
             annotator = annotators[a]
             try:
                 data = (annotator, annotator.get_annotation_data(context))
-            except TracError, e:
+            except TracError as e:
                 self.log.warning("Can't use annotator '%s': %s", a, e)
                 add_warning(context.req, tag.strong(
                     tag_("Can't use %(annotator)s annotator: %(error)s",
@@ -877,13 +861,12 @@ class Mimeview(Component):
             )
 
         def _body_rows():
-            for idx, line in enumerate(_group_lines(stream)):
+            for idx, line in enumerate(lines):
                 row = tag.tr()
-                if marks and idx + 1 in marks:
-                    row(class_='hilite')
                 for annotator, data in annotator_datas:
                     if annotator:
-                        annotator.annotate_row(context, row, idx+1, line, data)
+                        annotator.annotate_row(context, row, idx + 1,
+                                               line, data)
                     else:
                         row.append(tag.td())
                 row.append(tag.td(line))
@@ -894,11 +877,6 @@ class Mimeview(Component):
             tag.tbody(_body_rows())
         )
 
-    def get_max_preview_size(self):
-        """:deprecated: since 0.10, use `max_preview_size` attribute directly.
-        """
-        return self.max_preview_size
-
     def get_charset(self, content='', mimetype=None):
         """Infer the character encoding from the `content` or the `mimetype`.
 
@@ -954,7 +932,7 @@ class Mimeview(Component):
         charset = None
         if mimetype:
             charset = self.get_charset(content, mimetype)
-        if mimetype and charset and not 'charset' in mimetype:
+        if mimetype and charset and 'charset' not in mimetype:
             mimetype += '; charset=' + charset
         return mimetype
 
@@ -968,7 +946,7 @@ class Mimeview(Component):
                     mimetype, regexp = mapping.split(':', 1)
                 try:
                     self._mime_map_patterns[mimetype] = re.compile(regexp)
-                except re.error, e:
+                except re.error as e:
                     self.log.warning("mime_map_patterns contains invalid "
                                      "regexp '%s' for mimetype '%s' (%s)",
                                      regexp, mimetype, exception_to_unicode(e))
@@ -986,13 +964,6 @@ class Mimeview(Component):
             return True
         return False
 
-    def to_utf8(self, content, mimetype=None):
-        """Convert an encoded `content` to utf-8.
-
-        :deprecated: since 0.10, you should use `unicode` strings only.
-        """
-        return to_utf8(content, self.get_charset(content, mimetype))
-
     def to_unicode(self, content, mimetype=None, charset=None):
         """Convert `content` (an encoded `str` object) to an `unicode` object.
 
@@ -1071,82 +1042,6 @@ class Mimeview(Component):
         raise RequestDone
 
 
-def _group_lines(stream):
-    space_re = re.compile('(?P<spaces> (?: +))|^(?P<tag><\w+.*?>)?( )')
-
-    def pad_spaces(match):
-        m = match.group('spaces')
-        if m:
-            div, mod = divmod(len(m), 2)
-            return div * u'\xa0 ' + mod * u'\xa0'
-        return (match.group('tag') or '') + u'\xa0'
-
-    def _generate():
-        stack = []
-        def _reverse():
-            for event in reversed(stack):
-                if event[0] is START:
-                    yield END, event[1][0], event[2]
-                else:
-                    yield END_NS, event[1][0], event[2]
-
-        for kind, data, pos in stream:
-            if kind is TEXT:
-                lines = data.split('\n')
-                if lines:
-                    # First element
-                    for e in stack:
-                        yield e
-                    yield kind, lines.pop(0), pos
-                    for e in _reverse():
-                        yield e
-                    # Subsequent ones, prefix with \n
-                    for line in lines:
-                        yield TEXT, '\n', pos
-                        for e in stack:
-                            yield e
-                        yield kind, line, pos
-                        for e in _reverse():
-                            yield e
-            else:
-                if kind is START or kind is START_NS:
-                    stack.append((kind, data, pos))
-                elif kind is END or kind is END_NS:
-                    stack.pop()
-                else:
-                    yield kind, data, pos
-
-    buf = []
-
-    # Fix the \n at EOF.
-    if not isinstance(stream, list):
-        stream = list(stream)
-    found_text = False
-
-    for i in range(len(stream)-1, -1, -1):
-        if stream[i][0] is TEXT:
-            e = stream[i]
-            # One chance to strip a \n
-            if not found_text and e[1].endswith('\n'):
-                stream[i] = (e[0], e[1][:-1], e[2])
-            if len(e[1]):
-                found_text = True
-                break
-    if not found_text:
-        raise StopIteration
-
-    for kind, data, pos in _generate():
-        if kind is TEXT and data == '\n':
-            yield Stream(buf[:])
-            del buf[:]
-        else:
-            if kind is TEXT:
-                data = space_re.sub(pad_spaces, data)
-            buf.append((kind, data, pos))
-    if buf:
-        yield Stream(buf[:])
-
-
 # -- Default annotators
 
 class LineNumberAnnotator(Component):
@@ -1159,12 +1054,22 @@ class LineNumberAnnotator(Component):
         return 'lineno', _('Line'), _('Line numbers')
 
     def get_annotation_data(self, context):
-        return None
+        try:
+            marks = Ranges(context.get_hint('marks'))
+        except ValueError:
+            marks = None
+        return {
+            'id': context.get_hint('id', '') + 'L%s',
+            'marks': marks,
+            'offset': context.get_hint('lineno', 1) - 1
+        }
 
     def annotate_row(self, context, row, lineno, line, data):
-        row.append(tag.th(id='L%s' % lineno)(
-            tag.a(lineno, href='#L%s' % lineno)
-        ))
+        lineno += data['offset']
+        id = data['id'] % lineno
+        if data['marks'] and lineno in data['marks']:
+            row(class_='hilite')
+        row.append(tag.th(id=id)(tag.a(lineno, href='#' + id)))
 
 
 # -- Default renderers
