@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004 Oliver Rutherfurd
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin
 #         Oliver Rutherfurd (initial implementation)
@@ -36,8 +36,8 @@ try:
 except ImportError:
     has_docutils = False
 
+from trac.api import ISystemInfoProvider
 from trac.core import *
-from trac.env import ISystemInfoProvider
 from trac.mimeview.api import IHTMLPreviewRenderer, content_to_unicode
 from trac.util.html import Element, Fragment, Markup, escape, find_element
 from trac.util.translation import _
@@ -105,7 +105,7 @@ if has_docutils:
         ``[text]`` is optional.  If not given, ``target`` is
         used as the reference text.
 
-        .. _TracLink: http://trac.edgewall.org/wiki/TracLinks
+        .. _TracLink: https://trac.edgewall.org/wiki/TracLinks
         """
         if hasattr(state.inliner, 'trac'):
             env, context = state.inliner.trac
