@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # Copyright (C) 2008 Stephen Hansen
@@ -10,11 +10,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import hashlib
 import os
@@ -27,8 +27,6 @@ from email.mime.text import MIMEText
 from email.utils import formatdate, parseaddr, getaddresses
 from subprocess import Popen, PIPE
 
-from genshi.builder import tag
-
 from trac.config import (BoolOption, ConfigurationError, IntOption, Option,
                          OrderedExtensionsOption)
 from trac.core import Component, ExtensionPoint, TracError, implements
@@ -39,6 +37,7 @@ from trac.notification.api import (
 from trac.util import lazy
 from trac.util.compat import close_fds
 from trac.util.datefmt import time_now, to_utimestamp
+from trac.util.html import tag
 from trac.util.text import CRLF, exception_to_unicode, fix_eol, to_unicode
 from trac.util.translation import _, tag_
 from trac.web.session import get_session_attribute
@@ -49,7 +48,7 @@ __all__ = ['AlwaysEmailSubscriber', 'EMAIL_LOOKALIKE_PATTERN',
            'RecipientMatcher', 'SendmailEmailSender', 'SessionEmailResolver',
            'SmtpEmailSender', 'create_charset', 'create_header',
            'create_message_id', 'create_mime_multipart', 'create_mime_text',
-           'get_from_author', 'set_header']
+           'get_message_addresses', 'get_from_author', 'set_header']
 
 
 MAXHEADERLEN = 76
@@ -70,7 +69,7 @@ def create_charset(mime_encoding):
     """Create an appropriate email charset for the given encoding.
 
     Valid options are 'base64' for Base64 encoding, 'qp' for
-    Quoted-Printable, and 'none' for no encoding, in which case mails will
+    Quoted-Printable, and 'none' for no encoding, in which case emails will
     be sent as 7bit if the content is all ASCII, or 8bit otherwise.
     """
     charset = Charset()
@@ -94,36 +93,44 @@ def create_charset(mime_encoding):
     return charset
 
 
-def create_header(key, name, charset):
+def create_header(key, value, charset):
     """Create an appropriate email Header."""
     maxlength = MAXHEADERLEN-(len(key)+2)
-    # Do not sent ridiculous short headers
+    # Do not sent very short headers
     if maxlength < 10:
         raise TracError(_("Header length is too short"))
-    # when it matches mime-encoding, encode as mime even if only
-    # ascii characters
-    if not _mime_encoding_re.search(name):
-        try:
-            tmp = name.encode('ascii')
-            return Header(tmp, 'ascii', maxlinelen=maxlength)
-        except UnicodeEncodeError:
-            pass
-    return Header(name.encode(charset.output_codec), charset,
-                  maxlinelen=maxlength)
 
-
-def set_header(message, key, value, charset):
-    """Create and add or replace a header."""
     email = None
     if isinstance(value, (tuple, list)):
         value, email = value
     if not isinstance(value, basestring):
         value = to_unicode(value)
-    header = create_header(key, value, charset)
+    if not value:
+        return email
+
+    # when it matches mime-encoding, encode as mime even if only
+    # ascii characters
+    header = None
+    if not _mime_encoding_re.search(value):
+        try:
+            tmp = value.encode('ascii')
+        except UnicodeEncodeError:
+            pass
+        else:
+            header = Header(tmp, 'ascii', maxlinelen=maxlength)
+    if not header:
+        header = Header(value.encode(charset.output_codec), charset,
+                        maxlinelen=maxlength)
+    header = str(header)
     if email:
-        header = str(header).replace('\\', r'\\') \
-                            .replace('"', r'\"')
+        header = header.replace('\\', r'\\').replace('"', r'\"')
         header = '"%s" <%s>' % (header, email)
+    return header
+
+
+def set_header(message, key, value, charset):
+    """Create and add or replace a header."""
+    header = create_header(key, value, charset)
     if key in message:
         message.replace_header(key, header)
     else:
@@ -167,6 +174,10 @@ def create_message_id(env, targetid, from_email, time, more=None):
     return '<%03d.%s@%s>' % (len(source), h.hexdigest(), host)
 
 
+def get_message_addresses(message, name):
+    return getaddresses(str(header) for header in message.get_all(name, ()))
+
+
 def get_from_author(env, event):
     if event.author and NotificationSystem(env).smtp_from_author:
         matcher = RecipientMatcher(env)
@@ -189,12 +200,11 @@ class RecipientMatcher(object):
             domains = [domainfmt]
             domains.extend(re.escape(x) for x in admit_domains)
             addrfmt = r'%s@(?:%s)' % (localfmt, '|'.join(domains))
-        self.shortaddr_re = re.compile(r'(%s)$' % addrfmt, re.IGNORECASE)
+        self.shortaddr_re = re.compile(r'<?(%s)>?$' % addrfmt, re.IGNORECASE)
         self.longaddr_re = re.compile(r'(.*)\s+<\s*(%s)\s*>$' % addrfmt,
                                       re.IGNORECASE)
         self.ignore_domains = set(x.lower()
                                   for x in self.notify_sys.ignore_domains_list)
-        self.users = self.env.get_known_users(as_dict=True)
 
     @lazy
     def use_short_addr(self):
@@ -204,6 +214,10 @@ class RecipientMatcher(object):
     def smtp_default_domain(self):
         return self.notify_sys.smtp_default_domain
 
+    @lazy
+    def users(self):
+        return self.env.get_known_users(as_dict=True)
+
     def is_email(self, address):
         if not address:
             return False
@@ -227,7 +241,7 @@ class RecipientMatcher(object):
             auth = 0
             address = address.strip()
 
-        if not self.is_email(address) and self.nodomaddr_re.match(address):
+        if self.nodomaddr_re.match(address):
             if self.use_short_addr:
                 return sid, auth, address
             if self.smtp_default_domain:
@@ -238,12 +252,15 @@ class RecipientMatcher(object):
 
         mo = self.shortaddr_re.match(address)
         if mo:
-            return sid, auth, mo.group(1)
-        mo = self.longaddr_re.match(address)
-        if mo:
-            return sid, auth, mo.group(2)
-        self.env.log.debug("Invalid email address: %s", address)
-        return None
+            address = mo.group(1)
+        else:
+            mo = self.longaddr_re.match(address)
+            if mo:
+                address = mo.group(2)
+        if not self.is_email(address):
+            self.env.log.debug("Invalid email address: %s", address)
+            return None
+        return sid, auth, address
 
     def match_from_author(self, author):
         if author:
@@ -277,7 +294,7 @@ class EmailDistributor(Component):
         'SessionEmailResolver',
         include_missing=False,
         doc="""Comma separated list of email resolver components in the order
-        they will be called.  If an email address is resolved, the remaining
+        they will be called. If an email address is resolved, the remaining
         resolvers will not be called.
         """)
 
@@ -312,7 +329,7 @@ class EmailDistributor(Component):
             return
         self.log.debug("%s has found the following formats capable of "
                        "handling '%s' of '%s': %s", self.__class__.__name__,
-                       transport, event.realm, ', '.join(formats.keys()))
+                       transport, event.realm, ', '.join(formats))
 
         matcher = RecipientMatcher(self.env)
         notify_sys = NotificationSystem(self.env)
@@ -365,11 +382,11 @@ class EmailDistributor(Component):
             try:
                 outputs[fmt] = formatter.format(transport, fmt, event)
             except Exception as e:
-                self.log.warn('%s caught exception while '
-                              'formatting %s to %s for %s: %s%s',
-                              self.__class__.__name__, event.realm, fmt,
-                              transport, formatter.__class__,
-                              exception_to_unicode(e, traceback=True))
+                self.log.warning('%s caught exception while '
+                                 'formatting %s to %s for %s: %s%s',
+                                 self.__class__.__name__, event.realm, fmt,
+                                 transport, formatter.__class__,
+                                 exception_to_unicode(e, traceback=True))
                 failed.append(fmt)
 
         # Fallback to text/plain when formatter is broken
@@ -388,9 +405,9 @@ class EmailDistributor(Component):
                 bcc_addrs = sorted(addrs - always_cc)
                 self._do_send(transport, event, message, cc_addrs, bcc_addrs)
             else:
-                self.log.warn("%s cannot send event '%s' as '%s': %s",
-                              self.__class__.__name__, event.realm, fmt,
-                              ', '.join(addrs))
+                self.log.warning("%s cannot send event '%s' as '%s': %s",
+                                 self.__class__.__name__, event.realm, fmt,
+                                 ', '.join(addrs))
 
     def _create_message(self, format, outputs):
         if format not in outputs:
@@ -545,7 +562,7 @@ class SendmailEmailSender(Component):
         """Path to the sendmail executable.
 
         The sendmail program must accept the `-i` and `-f` options.
-         (''since 0.12'')""")
+        """)
 
     def send(self, from_addr, recipients, message):
         # Use native line endings in message
