@@ -1,34 +1,35 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
 import contextlib
 import copy
 import re
-
-from genshi.builder import tag
+from datetime import datetime
 
 from trac.cache import cached
 from trac.config import (
-    BoolOption, ConfigSection, ListOption, Option, OrderedExtensionsOption
-)
+    BoolOption, ConfigSection, IntOption, ListOption, Option,
+    OrderedExtensionsOption)
 from trac.core import *
 from trac.perm import IPermissionRequestor, PermissionCache, PermissionSystem
 from trac.resource import IResourceManager
 from trac.util import Ranges, as_bool, as_int
-from trac.util.text import shorten_line
+from trac.util.datefmt import parse_date, user_time
+from trac.util.html import tag
+from trac.util.text import shorten_line, to_unicode
 from trac.util.translation import _, N_, deactivate, gettext, reactivate
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
@@ -39,7 +40,7 @@ class TicketFieldList(list):
 
     def __init__(self, *args):
         super(TicketFieldList, self).__init__(*args)
-        self._map = dict((value['name'], value) for value in self)
+        self._map = {value['name']: value for value in self}
 
     def append(self, value):
         super(TicketFieldList, self).append(value)
@@ -171,6 +172,15 @@ class ITicketManipulator(Interface):
         detected. `field` can be `None` to indicate an overall problem with the
         ticket. Therefore, a return value of `[]` means everything is OK."""
 
+    def validate_comment(self, comment):
+        """Validate ticket comment.
+
+        Must return a list of messages, one for each problem detected.
+        The return value `[]` indicates no problems.
+
+        :since: 1.3.2
+        """
+
 
 class IMilestoneChangeListener(Interface):
     """Extension point interface for components that require notification
@@ -192,7 +202,8 @@ class IMilestoneChangeListener(Interface):
 
 
 class TicketSystem(Component):
-    implements(IPermissionRequestor, IWikiSyntaxProvider, IResourceManager)
+    implements(IPermissionRequestor, IWikiSyntaxProvider, IResourceManager,
+               ITicketManipulator)
 
     change_listeners = ExtensionPoint(ITicketChangeListener)
     milestone_change_listeners = ExtensionPoint(IMilestoneChangeListener)
@@ -248,7 +259,9 @@ class TicketSystem(Component):
         """Default keywords for newly created tickets.""")
 
     default_owner = Option('ticket', 'default_owner', '< default >',
-        """Default owner for newly created tickets.""")
+        """Default owner for newly created tickets. The component owner
+        is used when set to the value `< default >`.
+        """)
 
     default_cc = Option('ticket', 'default_cc', '',
         """Default cc: list for newly created tickets.""")
@@ -261,6 +274,15 @@ class TicketSystem(Component):
         """Comma-separated list of `select` fields that can have
         an empty value. (//since 1.1.2//)""")
 
+    max_comment_size = IntOption('ticket', 'max_comment_size', 262144,
+        """Maximum allowed comment size in characters.""")
+
+    max_description_size = IntOption('ticket', 'max_description_size', 262144,
+        """Maximum allowed description size in characters.""")
+
+    max_summary_size = IntOption('ticket', 'max_summary_size', 262144,
+        """Maximum allowed summary size in characters. (//since 1.0.2//)""")
+
     def __init__(self):
         self.log.debug('action controllers for ticket workflow: %r',
                        [c.__class__.__name__ for c in self.action_controllers])
@@ -292,8 +314,7 @@ class TicketSystem(Component):
 
     def get_ticket_field_labels(self):
         """Produce a (name,label) mapping from `get_ticket_fields`."""
-        labels = dict((f['name'], f['label'])
-                      for f in self.get_ticket_fields())
+        labels = {f['name']: f['label'] for f in self.get_ticket_fields()}
         labels['attachment'] = _("Attachment")
         return labels
 
@@ -308,7 +329,8 @@ class TicketSystem(Component):
         fields = copy.deepcopy(self.fields)
         label = 'label' # workaround gettext extraction bug
         for f in fields:
-            f[label] = gettext(f[label])
+            if not f.get('custom'):
+                f[label] = gettext(f[label])
         return fields
 
     def reset_ticket_fields(self):
@@ -410,6 +432,8 @@ class TicketSystem(Component):
             }
             if field['type'] == 'select' or field['type'] == 'radio':
                 field['options'] = config.getlist(name + '.options', sep='|')
+                if not field['options']:
+                    continue
                 if '' in field['options'] or \
                         field['name'] in self.allowed_empty_fields:
                     field['optional'] = True
@@ -419,8 +443,10 @@ class TicketSystem(Component):
                 field['value'] = '1' if as_bool(field['value']) else '0'
             elif field['type'] == 'text':
                 field['format'] = config.get(name + '.format', 'plain')
+                field['max_size'] = config.getint(name + '.max_size', 0)
             elif field['type'] == 'textarea':
                 field['format'] = config.get(name + '.format', 'plain')
+                field['max_size'] = config.getint(name + '.max_size', 0)
                 field['height'] = config.getint(name + '.rows')
             elif field['type'] == 'time':
                 field['format'] = config.get(name + '.format', 'datetime')
@@ -436,8 +462,7 @@ class TicketSystem(Component):
 
             fields.append(field)
 
-        fields.sort(lambda x, y: cmp((x['order'], x['name']),
-                                     (y['order'], y['name'])))
+        fields.sort(key=lambda f: (f['order'], f['name']))
         return fields
 
     def get_field_synonyms(self):
@@ -476,6 +501,74 @@ class TicketSystem(Component):
             allowed_owners.sort()
             return allowed_owners
 
+    # ITicketManipulator methods
+
+    def prepare_ticket(self, req, ticket, fields, actions):
+        pass
+
+    def validate_ticket(self, req, ticket):
+        # Validate select fields for known values.
+        for field in ticket.fields:
+            if 'options' not in field:
+                continue
+            name = field['name']
+            if name == 'status':
+                continue
+            if name in ticket and name in ticket._old:
+                value = ticket[name]
+                if value:
+                    if value not in field['options']:
+                        yield name, _('"%(value)s" is not a valid value',
+                                      value=value)
+                elif not field.get('optional', False):
+                    yield name, _("field cannot be empty")
+
+        # Validate description length.
+        if len(ticket['description'] or '') > self.max_description_size:
+            yield 'description', _("Must be less than or equal to %(num)s "
+                                   "characters",
+                                   num=self.max_description_size)
+
+        # Validate summary length.
+        if not ticket['summary']:
+            yield 'summary', _("Tickets must contain a summary.")
+        elif len(ticket['summary'] or '') > self.max_summary_size:
+            yield 'summary', _("Must be less than or equal to %(num)s "
+                               "characters", num=self.max_summary_size)
+
+        # Validate custom field length.
+        for field in ticket.custom_fields:
+            field_attrs = ticket.fields.by_name(field)
+            max_size = field_attrs.get('max_size', 0)
+            if 0 < max_size < len(ticket[field] or ''):
+                label = field_attrs.get('label')
+                yield label or field, _("Must be less than or equal to "
+                                        "%(num)s characters", num=max_size)
+
+        # Validate time field content.
+        for field in ticket.time_fields:
+            value = ticket[field]
+            if field in ticket.custom_fields and \
+                    field in ticket._old and \
+                    not isinstance(value, datetime):
+                field_attrs = ticket.fields.by_name(field)
+                format = field_attrs.get('format')
+                try:
+                    ticket[field] = user_time(req, parse_date, value,
+                                              hint=format) \
+                                    if value else None
+                except TracError as e:
+                    # Degrade TracError to warning.
+                    ticket[field] = value
+                    label = field_attrs.get('label')
+                    yield label or field, to_unicode(e)
+
+    def validate_comment(self, req, comment):
+        # Validate comment length
+        if len(comment or '') > self.max_comment_size:
+            yield _("Must be less than or equal to %(num)s characters",
+                    num=self.max_comment_size)
+
     # IPermissionRequestor methods
 
     def get_permission_actions(self):
