@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2009 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # Copyright (C) 2006 Alec Thomas
 # Copyright (C) 2007 Eli Carter
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
@@ -8,35 +8,35 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Eli Carter
 
+import io
 from ConfigParser import ParsingError, RawConfigParser
-from StringIO import StringIO
 from collections import defaultdict
 from functools import partial
 from pkg_resources import resource_filename
 
-from genshi.builder import tag
-
-from trac.config import Configuration, ConfigSection
+from trac.api import IEnvironmentSetupParticipant
+from trac.config import ConfigSection, Configuration, ConfigurationError
 from trac.core import *
-from trac.env import IEnvironmentSetupParticipant
 from trac.perm import PermissionCache, PermissionSystem
 from trac.resource import ResourceNotFound
 from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.ticket.model import Component as TicketComponent, Resolution
-from trac.util import get_reporter_id, sub_val, to_list
+from trac.util import exception_to_unicode, get_reporter_id, sub_val, to_list
+from trac.util.html import tag
 from trac.util.presentation import separated
 from trac.util.translation import _, tag_, cleandoc_
+from trac.versioncontrol.api import RepositoryManager
 from trac.web.chrome import Chrome, add_script, add_script_data
-from trac.wiki.formatter import system_message
-from trac.wiki.macros import WikiMacroBase
+from trac.wiki.formatter import MacroError, ProcessorError
+from trac.wiki.macros import WikiMacroBase, parse_args
 
 
 # -- Utilities for the ConfigurableTicketWorkflow
@@ -74,7 +74,7 @@ def parse_workflow_config(rawactions):
             actions[name]['newstate'] = newstate
         else:
             attribute = parts[1]
-            if attribute not in known_attrs.keys() or \
+            if attribute not in known_attrs or \
                     isinstance(known_attrs[attribute], str):
                 actions[name][attribute] = value
             elif isinstance(known_attrs[attribute], int):
@@ -130,6 +130,10 @@ class ConfigurableTicketWorkflow(Component):
         in the `trac.ini` file. See TracWorkflow for more details.
         """)
 
+    operations = ('del_owner', 'set_owner', 'set_owner_to_self',
+                  'may_set_owner', 'set_resolution', 'del_resolution',
+                  'leave_status', 'reset_workflow')
+
     def __init__(self):
         self.actions = self.get_all_actions()
         self.log.debug('Workflow actions at initialization: %s\n',
@@ -147,31 +151,10 @@ class ConfigurableTicketWorkflow(Component):
             self.actions = self.get_all_actions()
 
     def environment_needs_upgrade(self):
-        """The environment needs an upgrade if there is no [ticket-workflow]
-        section in the config.
-        """
-        return not list(self.config.options('ticket-workflow'))
+        pass
 
     def upgrade_environment(self):
-        """Insert a [ticket-workflow] section using the original-workflow"""
-        load_workflow_config_snippet(self.config, 'original-workflow.ini')
-        self.config.save()
-        self.actions = self.get_all_actions()
-        info_message = """
-
-==== Upgrade Notice ====
-
-The ticket Workflow is now configurable.
-
-Your environment has been upgraded, but configured to use the original
-workflow. It is recommended that you look at changing this configuration to use
-basic-workflow.
-
-Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
-
-"""
-        self.log.info(info_message.replace('\n', ' ').replace('==', ''))
-        print(info_message)
+        pass
 
     # ITicketActionController methods
 
@@ -184,34 +167,47 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         # the process of being modified, we need to base our information on the
         # pre-modified state so that we don't try to do two (or more!) steps at
         # once and get really confused.
-        status = ticket._old.get('status', ticket['status'])
-        exists = status is not None
+        ticket_status = ticket._old.get('status', ticket['status'])
+        exists = ticket_status is not None
+        ticket_owner = ticket._old.get('owner', ticket['owner'])
+        author = get_reporter_id(req, 'author')
 
-        ticket_perm = req.perm(ticket.resource)
+        resource = ticket.resource
         allowed_actions = []
         for action_name, action_info in self.actions.items():
+            operations = action_info['operations']
+            newstate = action_info['newstate']
+            # Exclude action that is effectively a No-op.
+            if len(operations) == 1 and \
+                    operations[0] == 'set_owner_to_self' and \
+                    ticket_owner == author and ticket_status == newstate:
+                continue
             oldstates = action_info['oldstates']
-            if exists and oldstates == ['*'] or status in oldstates:
+            if exists and oldstates == ['*'] or ticket_status in oldstates:
                 # This action is valid in this state.  Check permissions.
-                required_perms = action_info['permissions']
-                if self._is_action_allowed(ticket_perm, required_perms):
+                if self._is_action_allowed(req, action_info, resource):
                     allowed_actions.append((action_info['default'],
                                             action_name))
         # Append special `_reset` action if status is invalid.
-        if exists and status not in TicketSystem(self.env).get_all_status():
+        if exists and '_reset' in self.actions and \
+                ticket_status not in TicketSystem(self.env).get_all_status():
             reset = self.actions['_reset']
-            required_perms = reset['permissions']
-            if self._is_action_allowed(ticket_perm, required_perms):
+            if self._is_action_allowed(req, reset, resource):
                 allowed_actions.append((reset['default'], '_reset'))
         return allowed_actions
 
-    def _is_action_allowed(self, ticket_perm, required_perms):
-        if not required_perms:
-            return True
-        for permission in required_perms:
-            if permission in ticket_perm:
-                return True
-        return False
+    def _is_action_allowed(self, req, action, resource):
+        """Returns `True` if the workflow action is allowed for the `resource`.
+        """
+        perm_cache = req.perm(resource)
+        required_perms = action['permissions']
+        if required_perms:
+            for permission in required_perms:
+                if permission in perm_cache:
+                    break
+            else:
+                return False
+        return True
 
     def get_all_status(self):
         """Return a list of all states described by the configuration.
@@ -232,16 +228,19 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
 
         this_action = self.actions[action]
         status = this_action['newstate']
+        label = this_action['label']
         operations = this_action['operations']
-        current_owner = ticket._old.get('owner', ticket['owner'])
+        ticket_owner = ticket._old.get('owner', ticket['owner'])
+        ticket_status = ticket._old.get('status', ticket['status'])
         author = get_reporter_id(req, 'author')
         author_info = partial(Chrome(self.env).authorinfo, req,
                               resource=ticket.resource)
         format_author = partial(Chrome(self.env).format_author, req,
                                 resource=ticket.resource)
-        formatted_current_owner = author_info(current_owner)
-        exists = ticket._old.get('status', ticket['status']) is not None
+        formatted_current_owner = author_info(ticket_owner)
+        exists = ticket_status is not None
 
+        ticket_system = TicketSystem(self.env)
         control = []  # default to nothing
         hints = []
         if 'reset_workflow' in operations:
@@ -250,27 +249,15 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         if 'del_owner' in operations:
             hints.append(_("The ticket will be disowned"))
         if 'set_owner' in operations or 'may_set_owner' in operations:
-            if 'set_owner' in this_action:
-                owners = self._to_users(this_action['set_owner'], ticket)
-            elif self.config.getbool('ticket', 'restrict_owner'):
-                perm = PermissionSystem(self.env)
-                owners = perm.get_users_with_permission('TICKET_MODIFY')
-                owners = [user for user in owners
-                               if 'TICKET_MODIFY'
-                               in PermissionCache(self.env, user,
-                                                  ticket.resource)]
-                owners = sorted(owners)
-            else:
-                owners = None
+            owners = self.get_allowed_owners(req, ticket, this_action)
 
             if 'set_owner' in operations:
                 default_owner = author
             elif 'may_set_owner' in operations:
                 if not exists:
-                    default_owner = TicketSystem(self.env).default_owner
+                    default_owner = ticket_system.default_owner
                 else:
-                    default_owner = ticket._old.get('owner',
-                                                    ticket['owner'] or None)
+                    default_owner = ticket_owner or None
                 if owners is not None and default_owner not in owners:
                     owners.insert(0, default_owner)
             else:
@@ -286,7 +273,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                     tag_("to %(owner)s",
                          owner=tag.input(type='text', id=id, name=id,
                                          value=owner)))
-                if not exists or current_owner is None:
+                if not exists or ticket_owner is None:
                     hints.append(_("The owner will be the specified user"))
                 else:
                     hints.append(tag_("The owner will be changed from "
@@ -299,7 +286,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                 formatted_new_owner = author_info(owners[0])
                 control.append(tag_("to %(owner)s",
                                     owner=tag(formatted_new_owner, owner)))
-                if not exists or current_owner is None:
+                if not exists or ticket_owner is None:
                     hints.append(tag_("The owner will be %(new_owner)s",
                                       new_owner=formatted_new_owner))
                 elif ticket['owner'] != owners[0]:
@@ -310,37 +297,45 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
             else:
                 selected_owner = req.args.get(id, default_owner)
                 control.append(tag_("to %(owner)s", owner=tag.select(
-                    [tag.option(label, value=value if value is not None else '',
+                    [tag.option(text, value=value if value is not None else '',
                                 selected=(value == selected_owner or None))
-                     for label, value in sorted((format_author(owner), owner)
+                     for text, value in sorted((format_author(owner), owner)
                                                 for owner in owners)],
                     id=id, name=id)))
-                if not exists or current_owner is None:
+                if not exists or ticket_owner is None:
                     hints.append(_("The owner will be the selected user"))
                 else:
                     hints.append(tag_("The owner will be changed from "
                                       "%(current_owner)s to the selected user",
                                       current_owner=formatted_current_owner))
-        elif 'set_owner_to_self' in operations and \
-                ticket._old.get('owner', ticket['owner']) != author:
+        elif 'set_owner_to_self' in operations:
             formatted_author = author_info(author)
-            if not exists or current_owner is None:
+            if not exists or ticket_owner is None:
                 hints.append(tag_("The owner will be %(new_owner)s",
                                   new_owner=formatted_author))
-            else:
+            elif ticket_owner != author:
                 hints.append(tag_("The owner will be changed from "
                                   "%(current_owner)s to %(new_owner)s",
                                   current_owner=formatted_current_owner,
                                   new_owner=formatted_author))
+            elif ticket_status != status:
+                hints.append(tag_("The owner will remain %(current_owner)s",
+                                  current_owner=formatted_current_owner))
         if 'set_resolution' in operations:
+            resolutions = [r.name for r in Resolution.select(self.env)]
             if 'set_resolution' in this_action:
+                valid_resolutions = set(resolutions)
                 resolutions = this_action['set_resolution']
-            else:
-                resolutions = [r.name for r in Resolution.select(self.env)]
+                if any(x not in valid_resolutions for x in resolutions):
+                    raise ConfigurationError(_(
+                        "Your workflow attempts to set a resolution but uses "
+                        "undefined resolutions (configuration issue, please "
+                        "contact your Trac admin)."))
             if not resolutions:
-                raise TracError(_("Your workflow attempts to set a resolution "
-                                  "but none is defined (configuration issue, "
-                                  "please contact your Trac admin)."))
+                raise ConfigurationError(_(
+                    "Your workflow attempts to set a resolution but none is "
+                    "defined (configuration issue, please contact your Trac "
+                    "admin)."))
             id = 'action_%s_resolve_resolution' % action
             if len(resolutions) == 1:
                 resolution = tag.input(type='hidden', id=id, name=id,
@@ -352,7 +347,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                                   name=resolutions[0]))
             else:
                 selected_option = req.args.get(id,
-                        TicketSystem(self.env).default_resolution)
+                                               ticket_system.default_resolution)
                 control.append(tag_("as %(resolution)s",
                                     resolution=tag.select(
                     [tag.option(x, value=x,
@@ -363,30 +358,29 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         if 'del_resolution' in operations:
             hints.append(_("The resolution will be deleted"))
         if 'leave_status' in operations:
-            control.append(tag_("as %(status)s",
-                                status=ticket._old.get('status',
-                                                       ticket['status'])))
+            control.append(tag_("as %(status)s", status=ticket_status))
             if len(operations) == 1:
                 hints.append(tag_("The owner will remain %(current_owner)s",
                                   current_owner=formatted_current_owner)
-                             if current_owner else
+                             if ticket_owner else
                              _("The ticket will remain with no owner"))
-        elif not operations:
-            if status != '*':
-                if ticket['status'] is None:
-                    hints.append(tag_("The status will be '%(name)s'",
-                                      name=status))
-                else:
-                    hints.append(tag_("Next status will be '%(name)s'",
-                                      name=status))
-        return (this_action['label'], tag(separated(control, ' ')),
+        else:
+            if status == '*':
+                label = None  # Control won't be created
+            elif ticket['status'] is None:  # New ticket
+                hints.append(tag_("The status will be '%(name)s'",
+                                  name=status))
+            else:
+                hints.append(tag_("Next status will be '%(name)s'",
+                                  name=status))
+        return (label, tag(separated(control, ' ')),
                 tag(separated(hints, '. ', '.') if hints else ''))
 
     def get_ticket_changes(self, req, ticket, action):
         this_action = self.actions[action]
 
         # Enforce permissions
-        if not self._has_perms_for_action(req, this_action, ticket.resource):
+        if not self._is_action_allowed(req, this_action, ticket.resource):
             # The user does not have any of the listed permissions, so we won't
             # do anything.
             return {}
@@ -455,35 +449,28 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
     def apply_action_side_effects(self, req, ticket, action):
         pass
 
-    def _has_perms_for_action(self, req, action, resource):
-        required_perms = action['permissions']
-        if required_perms:
-            for permission in required_perms:
-                if permission in req.perm(resource):
-                    break
-            else:
-                # The user does not have any of the listed permissions
-                return False
-        return True
-
     # Public methods (for other ITicketActionControllers that want to use
     #                 our config file and provide an operation for an action)
 
     def get_all_actions(self):
         actions = parse_workflow_config(self.ticket_workflow_section.options())
 
-        # Special action that gets enabled if the current status no longer
-        # exists, as no other action can then change its state. (#5307/#11850)
-        reset = {
-            'default': 0,
-            'label': 'reset',
-            'newstate': 'new',
-            'oldstates': [],
-            'operations': ['reset_workflow'],
-            'permissions': ['TICKET_ADMIN']
-        }
-        for key, val in reset.items():
-            actions['_reset'].setdefault(key, val)
+        has_new_state = any('new' in [a['newstate']] + a['oldstates']
+                            for a in actions.itervalues())
+        if has_new_state:
+            # Special action that gets enabled if the current status no
+            # longer exists, as no other action can then change its state.
+            # (#5307/#11850)
+            reset = {
+                'default': 0,
+                'label': 'Reset',
+                'newstate': 'new',
+                'oldstates': [],
+                'operations': ['reset_workflow'],
+                'permissions': ['TICKET_ADMIN']
+            }
+            for key, val in reset.items():
+                actions['_reset'].setdefault(key, val)
 
         for name, info in actions.iteritems():
             for val in ('<none>', '< none >'):
@@ -516,9 +503,31 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                    if operation in info['operations'] and
                       ('*' in info['oldstates'] or
                        status in info['oldstates']) and
-                      self._has_perms_for_action(req, info, ticket.resource)]
+                      self._is_action_allowed(req, info, ticket.resource)]
         return actions
 
+    # Public methods
+
+    def get_allowed_owners(self, req, ticket, action):
+        """Returns users listed in the `set_owner` field of the action or
+        possessing the `TICKET_MODIFY` permission if `set_owner` is not
+        specified.
+
+        This method can be overridden in a subclasses in order to
+        customize the list of users that populate the assign-to select
+        box.
+
+        :since: 1.3.2
+        """
+        if 'set_owner' in action:
+            return self._to_users(action['set_owner'], ticket)
+        elif TicketSystem(self.env).restrict_owner:
+            users = PermissionSystem(self.env)\
+                    .get_users_with_permission('TICKET_MODIFY')
+            cache = partial(PermissionCache, self.env, resource=ticket.resource)
+            return sorted(u for u in users
+                            if 'TICKET_MODIFY' in cache(username=u))
+
     # Internal methods
 
     def _sub_owner_keyword(self, owner, ticket):
@@ -548,7 +557,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         def append_owners(users_perms_and_groups):
             for user_perm_or_group in users_perms_and_groups:
                 if user_perm_or_group == 'authenticated':
-                    owners.update(set(u[0] for u in self.env.get_known_users()))
+                    owners.update({u[0] for u in self.env.get_known_users()})
                 elif user_perm_or_group.isupper():
                     perm = user_perm_or_group
                     for user in ps.get_users_with_permission(perm):
@@ -573,72 +582,104 @@ class WorkflowMacro(WikiMacroBase):
 
     This macro accepts a TracWorkflow configuration and renders the states
     and transitions as a directed graph. If no parameters are given, the
-    current ticket workflow is rendered. In WikiProcessors mode the `width`
-    and `height` arguments can be specified.
+    current ticket workflow is rendered.
 
-    (Defaults: `width = 800` and `height = 600`)
+    In [WikiProcessors WikiProcessor] mode the `width` and `height`
+    arguments can be specified (Defaults: `width = 800` and `height = 600`).
+
+    The repository-scoped path of a workflow file can be specified as either
+    a macro or !WikiProcessor `file` argument. The file revision can be
+    specified by appending `@<rev>` to the path. The `file` argument value
+    must be enclosed in single or double quotes. //(Since 1.3.2)//.
 
     Examples:
     {{{
-        [[Workflow()]]
+    [[Workflow()]]
 
-        [[Workflow(go = here -> there; return = there -> here)]]
+    [[Workflow(go = here -> there; return = there -> here)]]
 
-        {{{
-        #!Workflow width=700 height=700
-        leave = * -> *
-        leave.operations = leave_status
-        leave.default = 1
+    [[Workflow(file=/contrib/workflow/enterprise-workflow.ini@1)]]
 
-        create = <none> -> new
-        create.default = 1
+    {{{#!Workflow file="/contrib/workflow/enterprise-workflow.ini"
+    }}}
 
-        create_and_assign = <none> -> assigned
-        create_and_assign.label = assign
-        create_and_assign.permissions = TICKET_MODIFY
-        create_and_assign.operations = may_set_owner
+    {{{#!Workflow width=700 height=700
+    leave = * -> *
+    leave.operations = leave_status
+    leave.default = 1
 
-        accept = new,assigned,accepted,reopened -> accepted
-        accept.permissions = TICKET_MODIFY
-        accept.operations = set_owner_to_self
+    create = <none> -> new
+    create.default = 1
 
-        resolve = new,assigned,accepted,reopened -> closed
-        resolve.permissions = TICKET_MODIFY
-        resolve.operations = set_resolution
+    create_and_assign = <none> -> assigned
+    create_and_assign.label = assign
+    create_and_assign.permissions = TICKET_MODIFY
+    create_and_assign.operations = may_set_owner
 
-        reassign = new,assigned,accepted,reopened -> assigned
-        reassign.permissions = TICKET_MODIFY
-        reassign.operations = set_owner
+    accept = new,assigned,accepted,reopened -> accepted
+    accept.permissions = TICKET_MODIFY
+    accept.operations = set_owner_to_self
 
-        reopen = closed -> reopened
-        reopen.permissions = TICKET_CREATE
-        reopen.operations = del_resolution
-        }}}
+    resolve = new,assigned,accepted,reopened -> closed
+    resolve.permissions = TICKET_MODIFY
+    resolve.operations = set_resolution
+
+    reassign = new,assigned,accepted,reopened -> assigned
+    reassign.permissions = TICKET_MODIFY
+    reassign.operations = set_owner
+
+    reopen = closed -> reopened
+    reopen.permissions = TICKET_CREATE
+    reopen.operations = del_resolution
+    }}}
     }}}
     """)
 
-    def expand_macro(self, formatter, name, text, args):
-        if not text:
+    def expand_macro(self, formatter, name, content, args=None):
+        if content is not None:
+            content = content.strip()
+        if not args and not content:
             raw_actions = self.config.options('ticket-workflow')
         else:
-            if args is None:
-                text = '\n'.join([line.lstrip() for line in text.split(';')])
+            is_macro = args is None
+            if is_macro:
+                kwargs = parse_args(content)[1]
+                file = kwargs.get('file')
+            else:
+                file = args.get('file')
+                if not file and not content:
+                    raise ProcessorError("Invalid argument(s).")
+
+            if file:
+                print(file)
+                text = RepositoryManager(self.env).read_file_by_path(file)
+                if text is None:
+                    raise ProcessorError(
+                        tag_("The file %(file)s does not exist.",
+                             file=tag.code(file)))
+            elif is_macro:
+                text = '\n'.join(line.lstrip() for line in content.split(';'))
+            else:
+                text = content
+
             if '[ticket-workflow]' not in text:
                 text = '[ticket-workflow]\n' + text
             parser = RawConfigParser()
             try:
-                parser.readfp(StringIO(text))
+                parser.readfp(io.StringIO(text))
             except ParsingError as e:
-                return system_message(_("Error parsing workflow."),
-                                      unicode(e))
+                if is_macro:
+                    raise MacroError(exception_to_unicode(e))
+                else:
+                    raise ProcessorError(exception_to_unicode(e))
             raw_actions = list(parser.items('ticket-workflow'))
         actions = parse_workflow_config(raw_actions)
-        states = list(set(
-            [state for action in actions.itervalues()
-                   for state in action['oldstates']] +
-            [action['newstate'] for action in actions.itervalues()]))
+        states = list(
+            {state for action in actions.itervalues()
+                   for state in action['oldstates']} |
+            {action['newstate'] for action in actions.itervalues()})
         action_labels = [attrs['label'] for attrs in actions.values()]
-        action_names = actions.keys()
+        action_names = list(actions)
         edges = []
         for name, action in actions.items():
             new_index = states.index(action['newstate'])
