@@ -1,42 +1,41 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2009 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from cStringIO import StringIO
 from datetime import datetime, timedelta
 from itertools import groupby
 import operator
 from math import ceil
 import csv
+import io
 import re
 
-from genshi.builder import tag
-
 from trac.config import Option, IntOption
 from trac.core import *
 from trac.db import get_column_names
 from trac.mimeview.api import IContentConverter, Mimeview
 from trac.resource import Resource
 from trac.ticket.api import TicketSystem, translation_deactivated
-from trac.ticket.model import Milestone
+from trac.ticket.model import Milestone, _datetime_to_db_str
 from trac.ticket.roadmap import group_milestones
 from trac.util import Ranges, as_bool
 from trac.util.datefmt import (datetime_now, from_utimestamp,
                                format_date_or_datetime, parse_date,
                                to_timestamp, to_utimestamp, utc, user_time)
+from trac.util.html import tag
 from trac.util.presentation import Paginator
 from trac.util.text import empty, shorten_line, quote_query_string
 from trac.util.translation import _, cleandoc_, ngettext, tag_
@@ -122,9 +121,9 @@ class Query(object):
         if verbose and 'description' not in rows:  # 0.10 compatibility
             rows.append('description')
         self.fields = TicketSystem(self.env).get_ticket_fields()
-        self.time_fields = set(f['name'] for f in self.fields
-                               if f['type'] == 'time')
-        field_names = set(f['name'] for f in self.fields)
+        self.time_fields = {f['name'] for f in self.fields
+                            if f['type'] == 'time'}
+        field_names = {f['name'] for f in self.fields}
         self.cols = [c for c in cols or [] if c in field_names or
                      c == 'id']
         self.rows = [c for c in rows if c in field_names]
@@ -240,8 +239,8 @@ class Query(object):
 
         # Semi-intelligently remove columns that are restricted to a single
         # value by a query constraint.
-        for col in [k for k in self.constraint_cols.keys()
-                    if k != 'id' and k in cols]:
+        for col in [k for k in self.constraint_cols
+                      if k != 'id' and k in cols]:
             constraints = self.constraint_cols[col]
             for constraint in constraints:
                 if not (len(constraint) == 1 and constraint[0]
@@ -270,6 +269,11 @@ class Query(object):
     def count(self, req=None, cached_ids=None, authname=None, tzinfo=None,
               locale=None):
         """Get the number of matching tickets for the present query.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
         """
         sql, args = self.get_sql(req, cached_ids, authname, tzinfo, locale)
         return self._count(sql, args)
@@ -284,6 +288,11 @@ class Query(object):
     def execute(self, req=None, cached_ids=None, authname=None, tzinfo=None,
                 href=None, locale=None):
         """Retrieve the list of matching tickets.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
         """
         if req is not None:
             href = req.href
@@ -312,11 +321,9 @@ class Query(object):
             columns = get_column_names(cursor)
             fields = [self.fields.by_name(column, None) for column in columns]
 
-            column_indices = range(len(columns))
             for row in cursor:
                 result = {}
-                for i in column_indices:
-                    name, field, val = columns[i], fields[i], row[i]
+                for name, field, val in zip(columns, fields, row):
                     if name == 'reporter':
                         val = val or 'anonymous'
                     elif name == 'id':
@@ -324,7 +331,7 @@ class Query(object):
                         if href is not None:
                             result['href'] = href.ticket(val)
                     elif name in self.time_fields:
-                        val = from_utimestamp(long(val)) if val else ''
+                        val = from_utimestamp(int(val)) if val else None
                     elif field and field['type'] == 'checkbox':
                         val = as_bool(val)
                     elif val is None:
@@ -406,11 +413,15 @@ class Query(object):
 
     def get_sql(self, req=None, cached_ids=None, authname=None, tzinfo=None,
                 locale=None):
-        """Return a (sql, params) tuple for the query."""
+        """Return a (sql, params) tuple for the query.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
+        """
         if req is not None:
             authname = req.authname
-            tzinfo = req.tz
-            locale = req.locale
         self.get_columns()
 
         # Build the list of actual columns to query
@@ -425,12 +436,14 @@ class Query(object):
         if self.rows:
             add_cols('reporter', *self.rows)
         add_cols('status', 'priority', 'time', 'changetime', self.order)
-        cols.extend([c for c in self.constraint_cols if c not in cols])
+        add_cols(*list(self.constraint_cols))
 
-        custom_fields = set(f['name'] for f in self.fields if f.get('custom'))
-        list_fields = set(f['name'] for f in self.fields
-                                    if f['type'] == 'text' and
-                                       f.get('format') == 'list')
+        custom_fields = {f['name'] for f in self.fields if f.get('custom')}
+        list_fields = {f['name'] for f in self.fields
+                                 if f['type'] == 'text' and
+                                    f.get('format') == 'list'}
+        cols_custom = [k for k in cols if k in custom_fields]
+        use_joins = len(cols_custom) <= 1
         enum_columns = [col for col in ('resolution', 'priority', 'severity',
                                         'type')
                             if col not in custom_fields and
@@ -438,9 +451,6 @@ class Query(object):
         joined_columns = [col for col in ('milestone', 'version')
                               if col not in custom_fields and
                                  col in (self.order, self.group)]
-        # 31 is max of joins in SQLite 32-bit
-        use_joins = (len(set(cols) & custom_fields) +
-                     len(enum_columns) + len(joined_columns)) <= 31
 
         sql = []
         sql.append("SELECT " + ",".join('t.%s AS %s' % (c, c) for c in cols
@@ -451,25 +461,25 @@ class Query(object):
         with self.env.db_query as db:
             if use_joins:
                 # Use LEFT OUTER JOIN for ticket_custom table
-                sql.extend(",%s.value AS %s" % ((db.quote(k),) * 2)
-                           for k in cols if k in custom_fields)
+                sql.extend(",%(qk)s.value AS %(qk)s" % {'qk': db.quote(k)}
+                           for k in cols_custom)
                 sql.append("\nFROM ticket AS t")
                 sql.extend("\n  LEFT OUTER JOIN ticket_custom AS %(qk)s ON "
                            "(%(qk)s.ticket=t.id AND %(qk)s.name='%(k)s')"
-                            % {'qk': db.quote(k), 'k': k}
-                            for k in cols if k in custom_fields)
+                            % {'qk': db.quote(k), 'k': k} for k in cols_custom)
             else:
-                # Use subquery for ticket_custom table
-                sql.extend(",%s AS %s" % ((db.quote(k),) * 2)
-                           for k in cols if k in custom_fields)
-                sql.append('\nFROM (\n  SELECT ')
-                sql.append(','.join('t.%s AS %s' % (c, c)
-                                    for c in cols if c not in custom_fields))
-                sql.extend(",\n  (SELECT c.value FROM ticket_custom c "
-                           "WHERE c.ticket=t.id AND c.name='%s') AS %s"
-                           % (k, db.quote(k))
-                           for k in cols if k in custom_fields)
-                sql.append("\n  FROM ticket AS t) AS t")
+                # Use MAX(CASE ... END) ... GROUP BY ... for ticket_custom
+                # table
+                sql.extend(",c.%(qk)s AS %(qk)s" % {'qk': db.quote(k)}
+                           for k in cols_custom)
+                sql.append("\nFROM ticket AS t"
+                           "\n  LEFT OUTER JOIN (SELECT\n    ticket AS id")
+                sql.extend(",\n    MAX(CASE WHEN name='%s' THEN value END) "
+                           "AS %s" % (k, db.quote(k)) for k in cols_custom)
+                sql.append("\n    FROM ticket_custom AS tc")
+                sql.append("\n    WHERE name IN (%s)" %
+                           ','.join("'%s'" % k for k in cols_custom))
+                sql.append("\n    GROUP BY tc.ticket) AS c ON c.id=t.id")
 
             # Join with the enum table for proper sorting
             sql.extend("\n  LEFT OUTER JOIN enum AS %(col)s ON "
@@ -482,46 +492,57 @@ class Query(object):
             sql.extend("\n  LEFT OUTER JOIN %(col)s ON (%(col)s.name=%(col)s)"
                        % {'col': col} for col in joined_columns)
 
-            def get_timestamp(date):
-                if date:
+            def user_parse_date(value):
+                if value:
                     try:
-                        return to_utimestamp(user_time(req, parse_date, date))
+                        return user_time(req, parse_date, value)
                     except TracError as e:
                         errors.append(unicode(e))
                 return None
 
             def get_constraint_sql(name, value, mode, neg):
-                if name not in custom_fields:
+                is_custom_field = name in custom_fields
+                if not is_custom_field:
                     col = 't.' + name
                 elif use_joins:
                     col = db.quote(name) + '.value'
                 else:
-                    col = 't.' + db.quote(name)
+                    col = 'c.' + db.quote(name)
                 value = value[len(mode) + neg:]
 
                 if name in self.time_fields:
+                    if not value:
+                        clause = "COALESCE({0},''){1}=%s" \
+                                 .format(col, '!' if neg else '')
+                        args = ['']
+                        return clause, args
                     if '..' in value:
                         (start, end) = [each.strip() for each in
                                         value.split('..', 1)]
                     else:
                         (start, end) = (value.strip(), '')
-                    col_cast = db.cast(col, 'int64')
-                    start = get_timestamp(start)
-                    end = get_timestamp(end)
+                    start = user_parse_date(start)
+                    end = user_parse_date(end)
+                    clause = args = None
                     if start is not None and end is not None:
-                        return ("%s(%s>=%%s AND %s<%%s)"
-                                % ('NOT ' if neg else '', col_cast, col_cast),
-                                (start, end))
+                        clause = "{0}({1}>=%s AND {1}<%s)" \
+                                 .format('NOT ' if neg else '', col)
+                        args = [start, end]
                     elif start is not None:
-                        return ("%s%s>=%%s"
-                                % ('NOT ' if neg else '', col_cast),
-                                (start, ))
+                        clause = "{0}{1}>=%s" \
+                                 .format('NOT ' if neg else '', col)
+                        args = [start]
                     elif end is not None:
-                        return ("%s%s<%%s"
-                                % ('NOT ' if neg else '', col_cast),
-                                (end, ))
+                        clause = "{0}{1}<%s" \
+                                 .format('NOT ' if neg else '', col)
+                        args = [end]
                     else:
                         return None
+                    if is_custom_field:
+                        args = [_datetime_to_db_str(arg, True) for arg in args]
+                    else:
+                        args = [to_utimestamp(arg) for arg in args]
+                    return clause, args
 
                 def split_words(splittable):
                     return [w.strip() for wl in
@@ -608,7 +629,7 @@ class Query(object):
                         elif use_joins:
                             col = db.quote(k) + '.value'
                         else:
-                            col = 't.' + db.quote(k)
+                            col = 'c.' + db.quote(k)
                         clauses.append("COALESCE(%s,'') %sIN (%s)"
                                        % (col, 'NOT ' if neg else '',
                                           ','.join('%s' for val in v)))
@@ -638,7 +659,7 @@ class Query(object):
                 sql.append(" OR ".join('(%s)' % c for c in clauses))
                 if cached_ids:
                     sql.append(" OR ")
-                    sql.append("id in (%s)" %
+                    sql.append("t.id in (%s)" %
                                (','.join(str(id) for id in cached_ids)))
 
             sql.append("\nORDER BY ")
@@ -654,15 +675,19 @@ class Query(object):
                 elif use_joins:
                     col = db.quote(name) + '.value'
                 else:
-                    col = 't.' + db.quote(name)
+                    col = 'c.' + db.quote(name)
                 desc = ' DESC' if desc else ''
                 # FIXME: This is a somewhat ugly hack.  Can we also have the
                 #        column type for this?  If it's an integer, we do
                 #        first one, if text, we do 'else'
-                if name == 'id' or name in self.time_fields:
-                    sql.append("COALESCE(%s,0)=0%s," % (col, desc))
+                if name in custom_fields:
+                    coalesce_arg = "''"
+                elif name == 'id' or name in self.time_fields:
+                    coalesce_arg = '0'
                 else:
-                    sql.append("COALESCE(%s,'')=''%s," % (col, desc))
+                    coalesce_arg = "''"
+                sql.append("COALESCE(%(col)s,%(arg)s)=%(arg)s%(desc)s," %
+                           {'col': col, 'arg': coalesce_arg, 'desc': desc})
                 if name in enum_columns:
                     # These values must be compared as ints, not as strings
                     sql.append(db.cast(col, 'int') + desc)
@@ -754,6 +779,7 @@ class Query(object):
                 field['options'].insert(0, {'name': '$USER',
                                             'value': '$USER'})
             if name == 'milestone' and not field.get('custom'):
+                field = field.copy()
                 milestones = [Milestone(self.env, opt)
                               for opt in field['options']]
                 milestones = [m for m in milestones
@@ -765,16 +791,22 @@ class Query(object):
                     for (label, milestones) in groups]
             fields[name] = field
 
+        def by_label(name):
+            if 'label' in fields[name]:
+                return fields[name]['label'].lower()
+            return ''
+        field_names = sorted(fields, key=by_label)
+
         groups = {}
         groupsequence = []
         for ticket in tickets:
-            if orig_list:
+            if orig_list and orig_time:
                 # Mark tickets added or changed since the query was first
                 # executed
-                if ticket['time'] > orig_time:
-                    ticket['added'] = True
-                elif ticket['changetime'] > orig_time:
-                    ticket['changed'] = True
+                if ticket['time'] and ticket['time'] > orig_time:
+                    ticket['_added'] = True
+                elif ticket['changetime'] and ticket['changetime'] > orig_time:
+                    ticket['_changed'] = True
             if self.group:
                 group_key = ticket[self.group]
                 groups.setdefault(group_key, []).append(ticket)
@@ -832,6 +864,7 @@ class Query(object):
                 'clauses': clauses,
                 'headers': headers,
                 'fields': fields,
+                'field_names': field_names,
                 'modes': self.get_modes(),
                 'tickets': tickets,
                 'groups': groupsequence or [(None, tickets)],
@@ -864,7 +897,7 @@ class QueryModule(Component):
 
     items_per_page = IntOption('query', 'items_per_page', 100,
         """Number of tickets displayed per page in ticket queries,
-        by default.
+        by default. Set to `0` to specify no limit.
         """)
 
     # IContentConverter methods
@@ -892,11 +925,8 @@ class QueryModule(Component):
         return 'tickets'
 
     def get_navigation_items(self, req):
-        from trac.ticket.report import ReportModule
         if 'TICKET_VIEW' in req.perm(self.realm) and \
-                (not self.env.is_component_enabled(ReportModule) or
-                 'REPORT_VIEW' not in req.perm(ReportModule.realm,
-                                               ReportModule.REPORT_LIST_ID)):
+                 'REPORT_VIEW' not in req.perm('report', -1):
             yield ('mainnav', 'tickets',
                    tag.a(_("View Tickets"), href=req.href.query()))
 
@@ -907,15 +937,15 @@ class QueryModule(Component):
 
     def process_request(self, req):
         req.perm(self.realm).assert_permission('TICKET_VIEW')
-        report_id = req.args.getfirst('report')
+        report_id = req.args.get('report')
         if report_id:
             req.perm('report', report_id).assert_permission('REPORT_VIEW')
 
         constraints = self._get_constraints(req)
         args = req.args
-        if not constraints and 'order' not in req.args:
+        if not constraints:
             # If no constraints are given in the URL, use the default ones.
-            if req.authname and req.authname != 'anonymous':
+            if req.is_authenticated:
                 qstring = self.default_query
                 user = req.authname
             else:
@@ -931,12 +961,12 @@ class QueryModule(Component):
                 constraints = self._get_constraints(arg_list=arg_list)
             else:
                 query = Query.from_string(self.env, qstring)
-                args = {'order': query.order, 'group': query.group,
-                        'col': query.cols, 'max': query.max}
-                if query.desc:
-                    args['desc'] = '1'
-                if query.groupdesc:
-                    args['groupdesc'] = '1'
+                args.setdefault('col', query.cols)
+                args.setdefault('desc', query.desc)
+                args.setdefault('group', query.group)
+                args.setdefault('groupdesc', query.groupdesc)
+                args.setdefault('max', query.max)
+                args.setdefault('order', query.order)
                 constraints = query.constraints
 
             # Substitute $USER, or ensure no field constraints that depend
@@ -950,33 +980,23 @@ class QueryModule(Component):
                             del clause[field]
                             break
 
-        cols = args.get('col')
-        if isinstance(cols, basestring):
-            cols = [cols]
+        cols = args.getlist('col')
         # Since we don't show 'id' as an option to the user,
         # we need to re-insert it here.
         if cols and 'id' not in cols:
             cols.insert(0, 'id')
-        rows = args.get('row', [])
-        if isinstance(rows, basestring):
-            rows = [rows]
+        rows = args.getlist('row')
         format = req.args.get('format')
         max = args.get('max')
         if max is None and format in ('csv', 'tab'):
             max = 0  # unlimited unless specified explicitly
         order = args.get('order')
-        if isinstance(order, (list, tuple)):
-            order = order[0] if order else None
         group = args.get('group')
-        if isinstance(group, (list, tuple)):
-            group = group[0] if group else None
         page = args.get('page')
-        if isinstance(page, (list, tuple)):
-            page = page[0] if page else None
         query = Query(self.env, report_id,
-                      constraints, cols, order, 'desc' in args, group,
-                      'groupdesc' in args, 'verbose' in args,
-                      rows, page, max)
+                      constraints, cols, order, as_bool(args.get('desc')),
+                      group, as_bool(args.get('groupdesc')),
+                      as_bool(args.get('verbose')), rows, page, max)
 
         if 'update' in req.args:
             # Reset session vars
@@ -1007,7 +1027,7 @@ class QueryModule(Component):
     def _get_constraints(self, req=None, arg_list=[]):
         fields = TicketSystem(self.env).get_ticket_fields()
         synonyms = TicketSystem(self.env).get_field_synonyms()
-        fields = dict((f['name'], f) for f in fields)
+        fields = {f['name']: f for f in fields}
         fields['id'] = {'type': 'id'}
         fields.update((k, fields[v]) for k, v in synonyms.iteritems())
 
@@ -1030,7 +1050,7 @@ class QueryModule(Component):
             # requested for clients without JavaScript
             add_num = None
             constraints = {}
-            for k, vals in req.args.iteritems():
+            for k in req.args:
                 match = self.add_re.match(k)
                 if match:
                     add_num = match.group(1)
@@ -1042,8 +1062,8 @@ class QueryModule(Component):
                 clause_num = int(match.group('clause'))
                 if field not in fields:
                     continue
-                if not isinstance(vals, (list, tuple)):
-                    vals = [vals]
+                # use list() to avoid modification of a list in req.args
+                vals = list(req.args.getlist(k))
                 if vals:
                     mode = req.args.get(k + '_mode')
                     if mode:
@@ -1055,7 +1075,7 @@ class QueryModule(Component):
                                     for (start, end) in zip(vals, ends)]
                     if k in remove_constraints:
                         idx = remove_constraints[k]
-                        if idx >= 0:
+                        if 0 <= idx < len(vals):
                             del vals[idx]
                             if not vals:
                                 continue
@@ -1067,7 +1087,7 @@ class QueryModule(Component):
             if add_num is not None:
                 field = req.args.get('add_filter_' + add_num,
                                      req.args.get('add_clause_' + add_num))
-                if field:
+                if field and field in fields:
                     clause = constraints.setdefault(int(add_num), {})
                     modes = Query.get_modes().get(fields[field]['type'])
                     mode = modes[0]['value'] if modes else ''
@@ -1125,10 +1145,8 @@ class QueryModule(Component):
         #
         # Note that with saved custom queries, there will be some convergence
         # between the report module and the query module.
-        from trac.ticket.report import ReportModule
-        report_resource = Resource(ReportModule.realm, query.id)
-        if 'REPORT_VIEW' in req.perm(report_resource) and \
-                self.env.is_component_enabled(ReportModule):
+        report_resource = Resource('report', query.id)
+        if 'REPORT_VIEW' in req.perm(report_resource):
             data['report_href'] = req.href.report()
             add_ctxtnav(req, _("Available Reports"), req.href.report())
             add_ctxtnav(req, _("New Custom Query"), req.href.query())
@@ -1150,30 +1168,22 @@ class QueryModule(Component):
         data['all_columns'].remove('id')
         data['all_textareas'] = query.get_all_textareas()
 
-        properties = dict((name, dict((key, field[key])
-                                      for key in ('type', 'label', 'options',
-                                                  'optgroups', 'optional',
-                                                  'format')
-                                      if key in field))
-                          for name, field in data['fields'].iteritems())
+        properties = {name: {key: field[key]
+                             for key in ('type', 'label', 'options',
+                                         'optgroups', 'optional', 'format')
+                             if key in field}
+                      for name, field in data['fields'].iteritems()}
         add_script_data(req, properties=properties, modes=data['modes'])
 
         add_stylesheet(req, 'common/css/report.css')
         Chrome(self.env).add_jquery_ui(req)
         add_script(req, 'common/js/query.js')
 
-        return 'query.html', data, None
-
-    def export_csv(self, req, query, sep=',', mimetype='text/plain'):
-        """:deprecated: since 1.0.6, use `_export_csv` instead. Will be
-                        removed in 1.3.1.
-        """
-        content, content_type = self._export_csv(req, query, sep, mimetype)
-        return ''.join(content), content_type
+        return 'query.html', data
 
     def _export_csv(self, req, query, sep=',', mimetype='text/plain'):
         def iterate():
-            out = StringIO()
+            out = io.BytesIO()
             writer = csv.writer(out, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
 
             def writerow(values):
@@ -1181,6 +1191,7 @@ class QueryModule(Component):
                                  for value in values])
                 rv = out.getvalue()
                 out.truncate(0)
+                out.seek(0)
                 return rv
 
             yield '\xef\xbb\xbf'  # BOM
@@ -1193,12 +1204,14 @@ class QueryModule(Component):
             chrome = Chrome(self.env)
             context = web_context(req)
             results = query.execute(req)
+            fields = dict((f['name'], f) for f in query.fields)
             for result in results:
                 ticket = Resource(self.realm, result['id'])
                 if 'TICKET_VIEW' in req.perm(ticket):
                     values = []
                     for col in cols:
                         value = result[col]
+                        field = fields.get(col)
                         if col in ('cc', 'owner', 'reporter'):
                             value = chrome.format_emails(context.child(ticket),
                                                          value)
@@ -1206,18 +1219,13 @@ class QueryModule(Component):
                             format = query.fields.by_name(col).get('format')
                             value = user_time(req, format_date_or_datetime,
                                               format, value) if value else ''
+                        elif field and field['type'] == 'checkbox':
+                            value = '1' if value else '0'
                         values.append(value)
                     yield writerow(values)
 
         return iterate(), '%s;charset=utf-8' % mimetype
 
-    def export_rss(self, req, query):
-        """:deprecated: since 1.0.6, use `_export_rss` instead. Will be
-                        removed in 1.3.1.
-        """
-        content, content_type = self._export_rss(req, query)
-        return ''.join(content), content_type
-
     def _export_rss(self, req, query):
         context = web_context(req, 'query', absurls=True)
         query_href = query.get_href(context.href)
@@ -1230,8 +1238,9 @@ class QueryModule(Component):
             'query_href': query_href
         }
         output = Chrome(self.env).render_template(req, 'query.rss', data,
-                                                  'application/rss+xml',
-                                                  iterable=True)
+                                                  {'content_type':
+                                                   'application/rss+xml',
+                                                   'iterable': True})
         return output, 'application/rss+xml'
 
     # IWikiSyntaxProvider methods
@@ -1401,8 +1410,8 @@ class TicketQueryMacro(WikiMacroBase):
 
             add_stylesheet(req, 'common/css/report.css')
 
-            return Chrome(self.env).render_template(
-                req, 'query_results.html', data, None, fragment=True)
+            return Chrome(self.env).render_fragment(req, 'query_results.html',
+                                                    data)
 
         if format == 'progress':
             from trac.ticket.roadmap import (RoadmapModule,
@@ -1444,8 +1453,7 @@ class TicketQueryMacro(WikiMacroBase):
                     'legend': True,
                 }
                 return tag.div(
-                    chrome.render_template(req, 'progress_bar.html', data,
-                                           None, fragment=True),
+                    chrome.render_fragment(req, 'progress_bar.html', data),
                     class_='trac-progress')
 
             def per_group_stats_data(gstat, group_name):
@@ -1470,8 +1478,7 @@ class TicketQueryMacro(WikiMacroBase):
                              group=by),
             }
             return tag.div(
-                chrome.render_template(req, 'progress_bar_grouped.html', data,
-                                       None, fragment=True),
+                chrome.render_fragment(req, 'progress_bar_grouped.html', data),
                 class_='trac-groupprogress')
 
         # Formats above had their own permission checks, here we need to
