@@ -1,27 +1,25 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from StringIO import StringIO
 from datetime import datetime, timedelta
+import io
 import itertools
 import re
 
-from genshi.builder import tag
-
 from trac.attachment import Attachment, AttachmentModule
 from trac.config import ConfigSection, ExtensionOption, Option
 from trac.core import *
@@ -34,6 +32,7 @@ from trac.util.datefmt import (datetime_now, format_date, format_datetime,
                                from_utimestamp, get_datetime_format_hint,
                                parse_date, pretty_timedelta, to_datetime,
                                user_time, utc)
+from trac.util.html import tag
 from trac.util.presentation import classes
 from trac.util.text import CRLF, exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
@@ -42,20 +41,22 @@ from trac.ticket.notification import BatchTicketChangeEvent
 from trac.ticket.model import Milestone, MilestoneCache, Ticket
 from trac.timeline.api import ITimelineEventProvider
 from trac.web.api import HTTPBadRequest, IRequestHandler, RequestDone
-from trac.web.chrome import (Chrome, INavigationContributor,
-                             add_link, add_notice, add_script, add_stylesheet,
-                             add_warning, auth_link, prevnext_nav, web_context)
+from trac.web.chrome import (Chrome, INavigationContributor, accesskey,
+                             add_link, add_notice, add_stylesheet, add_warning,
+                             auth_link, prevnext_nav, web_context)
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.formatter import format_to
 
 
 class ITicketGroupStatsProvider(Interface):
+
     def get_ticket_group_stats(ticket_ids):
         """ Gather statistics on a group of tickets.
 
         This method returns a valid `TicketGroupStats` object.
         """
 
+
 class TicketGroupStats(object):
     """Encapsulates statistics on a group of tickets."""
 
@@ -120,9 +121,9 @@ class TicketGroupStats(object):
                 self.done_count += interval['count']
 
         # We want the percentages to add up to 100%. To do that, we fudge one
-        # of the intervals. If we're <100%, we add to the smallest non-zero
-        # interval. If we're >100%, we subtract from the largest interval.
-        # The interval is adjusted by enough to make the intervals sum to 100%.
+        # of the intervals. If we're < 100%, we add to the smallest non-zero
+        # interval. If we're > 100%, we subtract from the largest interval.
+        # The interval is adjusted to make the intervals sum to 100%.
         if self.done_count and total_percent != 100:
             fudge_amt = 100 - total_percent
             fudge_int = [i for i in sorted(self.intervals,
@@ -165,7 +166,7 @@ class DefaultTicketGroupStatsProvider(Component):
         closed.order = 0
 
         # .query_args: optional parameters for the corresponding
-        #              query.  In this example, the changes from the
+        #              query. In this example, the changes from the
         #              default are two additional columns ('created' and
         #              'modified'), and sorting is done on 'created'.
         closed.query_args = group=resolution,order=time,col=id,col=summary,col=owner,col=type,col=priority,col=component,col=severity,col=time,col=changetime
@@ -191,7 +192,7 @@ class DefaultTicketGroupStatsProvider(Component):
         }}}
 
         The definition consists in a comma-separated list of accepted
-        status.  Also, '*' means any status and could be used to
+        status. Also, '*' means any status and could be used to
         associate all remaining states to one catch-all group.
 
         The CSS class can be one of: new (yellow), open (no color) or
@@ -201,11 +202,11 @@ class DefaultTicketGroupStatsProvider(Component):
         for example.
         """)
 
-    default_milestone_groups =  [
+    default_milestone_groups = [
         {'name': 'closed', 'status': 'closed',
          'query_args': 'group=resolution', 'overall_completion': 'true'},
         {'name': 'active', 'status': '*', 'css_class': 'open'}
-        ]
+    ]
 
     def _get_ticket_groups(self):
         """Returns a list of dict describing the ticket groups
@@ -240,9 +241,9 @@ class DefaultTicketGroupStatsProvider(Component):
                     """ % ",".join(str(x) for x in sorted(ticket_ids))):
                 status_cnt[status] = count
 
-        stat = TicketGroupStats(_('ticket status'), _('tickets'))
+        stat = TicketGroupStats(_("ticket status"), _("tickets"))
         remaining_statuses = set(all_statuses)
-        groups =  self._get_ticket_groups()
+        groups = self._get_ticket_groups()
         catch_all_group = None
         # we need to go through the groups twice, so that the catch up group
         # doesn't need to be the last one in the sequence
@@ -257,9 +258,8 @@ class DefaultTicketGroupStatsProvider(Component):
                         group1=group['name'], group2=catch_all_group['name']))
                 catch_all_group = group
             else:
-                group_statuses = set([s.strip()
-                                      for s in status_str.split(',')]) \
-                                      & all_statuses
+                group_statuses = {s.strip() for s in status_str.split(',')} \
+                                 & all_statuses
                 if group_statuses - remaining_statuses:
                     raise TracError(_(
                         "'%(groupname)s' milestone group reused status "
@@ -301,12 +301,12 @@ def get_tickets_for_milestone(env, milestone=None, field='component'):
     fields = TicketSystem(env).get_ticket_fields()
     if field in [f['name'] for f in fields if not f.get('custom')]:
         sql = """SELECT id, status, %s FROM ticket WHERE milestone=%%s
-                 ORDER BY %s""" % (field, field)
+                 ORDER BY %s, id""" % (field, field)
         args = (milestone,)
     else:
         sql = """SELECT id, status, value FROM ticket
                    LEFT OUTER JOIN ticket_custom ON (id=ticket AND name=%s)
-                  WHERE milestone=%s ORDER BY value"""
+                  WHERE milestone=%s ORDER BY value, id"""
         args = (field, milestone)
     return [{'id': tkt_id, 'status': status, field: fieldval}
             for tkt_id, status, fieldval in env.db_query(sql, args)]
@@ -361,6 +361,7 @@ def apply_ticket_permissions(env, req, tickets):
     return [t for t in tickets
             if 'TICKET_VIEW' in req.perm('ticket', t['id'])]
 
+
 def milestone_stats_data(env, req, stat, name, grouped_by='component',
                          group=None):
     from trac.ticket.query import QueryModule
@@ -376,6 +377,7 @@ def milestone_stats_data(env, req, stat, name, grouped_by='component',
             'interval_hrefs': [query_href(interval['qry_args'])
                                for interval in stat.intervals]}
 
+
 def grouped_stats_data(env, stats_provider, tickets, by, per_group_stats_data):
     """Get the `tickets` stats data grouped by ticket field `by`.
 
@@ -445,7 +447,7 @@ def group_milestones(milestones, include_completed):
 
 
 class RoadmapModule(Component):
-    """Give an overview over all the milestones."""
+    """Overview of all the milestones."""
 
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler)
 
@@ -464,7 +466,8 @@ class RoadmapModule(Component):
     def get_navigation_items(self, req):
         if 'ROADMAP_VIEW' in req.perm:
             yield ('mainnav', 'roadmap',
-                   tag.a(_('Roadmap'), href=req.href.roadmap(), accesskey=3))
+                   tag.a(_('Roadmap'), href=req.href.roadmap(),
+                         accesskey=accesskey(req, 3)))
 
     # IPermissionRequestor methods
 
@@ -502,7 +505,7 @@ class RoadmapModule(Component):
             stat = get_ticket_stats(self.stats_provider, tickets)
             stats.append(milestone_stats_data(self.env, req, stat,
                                               milestone.name))
-            #milestone['tickets'] = tickets # for the iCalendar view
+            # milestone['tickets'] = tickets  # for the iCalendar view
 
         if req.args.get('format') == 'ics':
             self._render_ics(req, milestones)
@@ -510,10 +513,10 @@ class RoadmapModule(Component):
 
         # FIXME should use the 'webcal:' scheme, probably
         username = None
-        if req.authname and req.authname != 'anonymous':
+        if req.is_authenticated:
             username = req.authname
         icshref = req.href.roadmap(show=show, user=username, format='ics')
-        add_link(req, 'alternate', auth_link(req, icshref), _('iCalendar'),
+        add_link(req, 'alternate', auth_link(req, icshref), _("iCalendar"),
                  'text/calendar', 'ics')
 
         data = {
@@ -523,19 +526,20 @@ class RoadmapModule(Component):
             'show': show,
         }
         add_stylesheet(req, 'common/css/roadmap.css')
-        return 'roadmap.html', data, None
+        return 'roadmap.html', data
 
     # Internal methods
 
     def _render_ics(self, req, milestones):
         req.send_response(200)
         req.send_header('Content-Type', 'text/calendar;charset=utf-8')
-        buf = StringIO()
+        buf = io.StringIO()
 
         from trac.ticket import Priority
         priorities = {}
         for priority in Priority.select(self.env):
             priorities[priority.name] = float(priority.value)
+
         def get_priority(ticket):
             value = priorities.get(ticket['priority'])
             if value:
@@ -544,7 +548,8 @@ class RoadmapModule(Component):
 
         def get_status(ticket):
             status = ticket['status']
-            if status == 'new' or status == 'reopened' and not ticket['owner']:
+            if status == 'new' or status == 'reopened' and \
+                    not ticket['owner']:
                 return 'NEEDS-ACTION'
             elif status in ('assigned', 'reopened'):
                 return 'IN-PROCESS'
@@ -553,16 +558,19 @@ class RoadmapModule(Component):
                     return 'COMPLETED'
                 else:
                     return 'CANCELLED'
-            else: return ''
+            else:
+                return ''
 
         def escape_value(text):
             s = ''.join(map(lambda c: '\\' + c if c in ';,\\' else c, text))
             return '\\n'.join(re.split(r'[\r\n]+', s))
 
         def write_prop(name, value, params={}):
-            text = ';'.join([name] + [k + '=' + v for k, v in params.items()]) \
-                 + ':' + escape_value(value)
+            text = ';'.join([name] + [k + '=' + v for k, v
+                                                  in params.items()]) + \
+                   ':' + escape_value(value)
             firstline = 1
+            text = to_unicode(text)
             while text:
                 if not firstline:
                     text = ' ' + text
@@ -585,10 +593,10 @@ class RoadmapModule(Component):
         write_prop('BEGIN', 'VCALENDAR')
         write_prop('VERSION', '2.0')
         write_prop('PRODID', '-//Edgewall Software//NONSGML Trac %s//EN'
-                   % self.env.trac_version)
+                             % self.env.trac_version)
         write_prop('METHOD', 'PUBLISH')
         write_prop('X-WR-CALNAME',
-                   self.env.project_name + ' - ' + _('Roadmap'))
+                   self.env.project_name + ' - ' + _("Roadmap"))
         write_prop('X-WR-CALDESC', self.env.project_description)
         write_prop('X-WR-TIMEZONE', str(req.tz))
 
@@ -601,7 +609,7 @@ class RoadmapModule(Component):
                 write_prop('UID', uid)
                 write_utctime('DTSTAMP', milestone.due)
                 write_date('DTSTART', milestone.due)
-                write_prop('SUMMARY', _('Milestone %(name)s',
+                write_prop('SUMMARY', _("Milestone %(name)s",
                                         name=milestone.name))
                 write_prop('URL', req.abs_href.milestone(milestone.name))
                 if milestone.description:
@@ -618,7 +626,7 @@ class RoadmapModule(Component):
                 if milestone.due:
                     write_prop('RELATED-TO', uid)
                     write_date('DUE', milestone.due)
-                write_prop('SUMMARY', _('Ticket #%(num)s: %(summary)s',
+                write_prop('SUMMARY', _("Ticket #%(num)s: %(summary)s",
                                         num=ticket.id,
                                         summary=ticket['summary']))
                 write_prop('URL', req.abs_href.ticket(ticket.id))
@@ -687,7 +695,7 @@ class MilestoneModule(Component):
 
     def get_timeline_filters(self, req):
         if 'MILESTONE_VIEW' in req.perm:
-            yield ('milestone', _('Milestones completed'))
+            yield ('milestone', _("Milestones completed"))
 
     def get_timeline_events(self, req, start, stop, filters):
         if 'milestone' in filters:
@@ -699,12 +707,12 @@ class MilestoneModule(Component):
                     #       reported
                     milestone = milestone_realm(id=name)
                     if 'MILESTONE_VIEW' in req.perm(milestone):
-                        yield ('milestone', completed, '', # FIXME: author?
+                        yield ('milestone', completed, '',  # FIXME: author?
                                (milestone, description))
 
             # Attachments
             for event in AttachmentModule(self.env).get_timeline_events(
-                req, milestone_realm, start, stop):
+                    req, milestone_realm, start, stop):
                 yield event
 
     def render_timeline_event(self, context, field, event):
@@ -712,11 +720,11 @@ class MilestoneModule(Component):
         if field == 'url':
             return context.href.milestone(milestone.id)
         elif field == 'title':
-            return tag_('Milestone %(name)s completed',
+            return tag_("Milestone %(name)s completed",
                         name=tag.em(milestone.id))
         elif field == 'description':
-            return format_to(self.env, None, context.child(resource=milestone),
-                             description)
+            child_resource = context.child(resource=milestone)
+            return format_to(self.env, None, child_resource, description)
 
     # IRequestHandler methods
 
@@ -729,11 +737,13 @@ class MilestoneModule(Component):
 
     def process_request(self, req):
         milestone_id = req.args.get('id')
+        action = req.args.get('action', 'view')
+        if not milestone_id and action == 'view':
+            req.redirect(req.href.roadmap())
         req.perm(self.realm, milestone_id).require('MILESTONE_VIEW')
 
-        add_link(req, 'up', req.href.roadmap(), _('Roadmap'))
+        add_link(req, 'up', req.href.roadmap(), _("Roadmap"))
 
-        action = req.args.get('action', 'view')
         try:
             milestone = Milestone(self.env, milestone_id)
         except ResourceNotFound:
@@ -741,7 +751,7 @@ class MilestoneModule(Component):
                 raise
             milestone = Milestone(self.env)
             milestone.name = milestone_id
-            action = 'edit' # rather than 'new' so that it works for POST/save
+            action = 'edit'  # rather than 'new', so it works for POST/save
 
         if req.method == 'POST':
             if 'cancel' in req.args:
@@ -779,8 +789,9 @@ class MilestoneModule(Component):
         return to_datetime(default_due, req.tz)
 
     def save_milestone(self, req, milestone):
-        # Instead of raising one single error, check all the constraints and
-        # let the user fix them by going back to edit mode showing the warnings
+        # Instead of raising one single error, check all the constraints
+        # and let the user fix them by going back to edit mode and showing
+        # the warnings
         warnings = []
         def warn(msg):
             add_warning(req, msg)
@@ -802,7 +813,7 @@ class MilestoneModule(Component):
             completed = user_time(req, parse_date, completed,
                                   hint='datetime') if completed else None
             if completed and completed > datetime_now(utc):
-                warn(_('Completion date may not be in the future'))
+                warn(_("Completion date may not be in the future"))
         else:
             completed = None
         milestone.completed = completed
@@ -833,10 +844,10 @@ class MilestoneModule(Component):
         if milestone.exists:
             try:
                 milestone.update(author=req.authname)
-            except self.env.db_exc.IntegrityError:
-                raise TracError(_('Milestone "%(name)s" already exists, '
-                                  'please choose another name.',
-                                  name=milestone.name))
+            except ResourceExistsError:
+                raise ResourceExistsError(_('Milestone "%(name)s" already '
+                                            'exists, please choose another '
+                                            'name.', name=milestone.name))
             if completed and 'retarget' in req.args:
                 comment = req.args.get('comment', '')
                 retarget_to = req.args.get('target') or None
@@ -867,10 +878,10 @@ class MilestoneModule(Component):
         else:
             try:
                 milestone.insert()
-            except self.env.db_exc.IntegrityError:
-                raise TracError(_('Milestone "%(name)s" already exists, '
-                                  'please choose another name.',
-                                  name=milestone.name))
+            except ResourceExistsError:
+                raise ResourceExistsError(_('Milestone "%(name)s" already '
+                                            'exists, please choose another '
+                                            'name.', name=milestone.name))
             add_notice(req, _('The milestone "%(name)s" has been added.',
                               name=milestone.name))
 
@@ -885,9 +896,10 @@ class MilestoneModule(Component):
         if self._default_retarget_to and \
            not any(self._default_retarget_to == m.name
                    for m in Milestone.select(self.env)):
-            self.log.warn('Milestone "%s" does not exist. Update the '
-                          '"default_retarget_to" option in the [milestone] '
-                          'section of trac.ini', self._default_retarget_to)
+            self.log.warning('Milestone "%s" does not exist. Update the '
+                             '"default_retarget_to" option in the '
+                             '[milestone] section of trac.ini',
+                             self._default_retarget_to)
         return self._default_retarget_to
 
     def _do_delete(self, req, milestone):
@@ -898,7 +910,7 @@ class MilestoneModule(Component):
         retargeted_tickets = \
             milestone.move_tickets(retarget_to, req.authname,
                 "Ticket retargeted after milestone deleted")
-        milestone.delete(author=req.authname)
+        milestone.delete()
         add_notice(req, _('The milestone "%(name)s" has been deleted.',
                           name=milestone.name))
         if retargeted_tickets:
@@ -943,14 +955,14 @@ class MilestoneModule(Component):
         attachments = Attachment.select(self.env, self.realm, milestone.name)
         data = {
             'milestone': milestone,
-            'milestone_groups': group_milestones(milestones,
-                'TICKET_ADMIN' in req.perm),
+            'milestone_groups':
+                group_milestones(milestones, 'TICKET_ADMIN' in req.perm),
             'num_tickets': get_num_tickets_for_milestone(self.env, milestone),
             'retarget_to': self.default_retarget_to,
             'attachments': list(attachments)
         }
         add_stylesheet(req, 'common/css/roadmap.css')
-        return 'milestone_delete.html', data, None
+        return 'milestone_delete.html', data
 
     def _render_editor(self, req, milestone):
         data = {
@@ -965,8 +977,8 @@ class MilestoneModule(Component):
             milestones = [m for m in Milestone.select(self.env)
                           if m.name != milestone.name
                           and 'MILESTONE_VIEW' in req.perm(m.resource)]
-            data['milestone_groups'] = group_milestones(milestones,
-                'TICKET_ADMIN' in req.perm)
+            data['milestone_groups'] = \
+                group_milestones(milestones, 'TICKET_ADMIN' in req.perm)
             data['num_open_tickets'] = \
                 get_num_tickets_for_milestone(self.env, milestone,
                                               exclude_closed=True)
@@ -974,14 +986,15 @@ class MilestoneModule(Component):
         else:
             req.perm(milestone.resource).require('MILESTONE_CREATE')
             if milestone.name:
-                add_notice(req, _("Milestone %(name)s does not exist. You can"
-                                  " create it here.", name=milestone.name))
+                add_notice(req, _("Milestone %(name)s does not exist. You "
+                                  "can create it here.", name=milestone.name))
 
         chrome = Chrome(self.env)
         chrome.add_jquery_ui(req)
         chrome.add_wiki_toolbars(req)
+        chrome.add_auto_preview(req)
         add_stylesheet(req, 'common/css/roadmap.css')
-        return 'milestone_edit.html', data, None
+        return 'milestone_edit.html', data
 
     def _render_view(self, req, milestone):
         milestone_groups = []
@@ -1006,7 +1019,8 @@ class MilestoneModule(Component):
             by = available_groups[0]['name']
         by = req.args.getfirst('by', by)
 
-        tickets = get_tickets_for_milestone(self.env, milestone=milestone.name,
+        tickets = get_tickets_for_milestone(self.env,
+                                            milestone=milestone.name,
                                             field=by)
         tickets = apply_ticket_permissions(self.env, req, tickets)
         stat = get_ticket_stats(self.stats_provider, tickets)
@@ -1015,13 +1029,12 @@ class MilestoneModule(Component):
         data = {
             'context': context,
             'milestone': milestone,
-            'attachments': AttachmentModule(self.env).attachment_data(context),
+            'attachments':
+                AttachmentModule(self.env).attachment_data(context),
             'available_groups': available_groups,
             'grouped_by': by,
-            'groups': milestone_groups,
-            'summary': _("Shows the milestone completion status grouped by "
-                         "%(grouped_by)s", grouped_by=by),
-            }
+            'groups': milestone_groups
+        }
         data.update(milestone_stats_data(self.env, req, stat, milestone.name))
 
         if by:
@@ -1029,11 +1042,10 @@ class MilestoneModule(Component):
                 return milestone_stats_data(self.env, req, gstat,
                                             milestone.name, by, group_name)
             milestone_groups.extend(
-                grouped_stats_data(self.env, self.stats_provider, tickets, by,
-                                   per_group_stats_data))
+                grouped_stats_data(self.env, self.stats_provider, tickets,
+                                   by, per_group_stats_data))
 
         add_stylesheet(req, 'common/css/roadmap.css')
-        add_script(req, 'common/js/folding.js')
 
         def add_milestone_link(rel, milestone):
             href = req.href.milestone(milestone.name, by=req.args.get('by'))
@@ -1051,10 +1063,10 @@ class MilestoneModule(Component):
             if idx < len(milestones) - 1:
                 add_milestone_link('next', milestones[idx + 1])
                 add_milestone_link('last', milestones[-1])
-        prevnext_nav(req, _('Previous Milestone'), _('Next Milestone'),
-                     _('Back to Roadmap'))
+        prevnext_nav(req, _("Previous Milestone"), _("Next Milestone"),
+                     _("Back to Roadmap"))
 
-        return 'milestone_view.html', data, None
+        return 'milestone_view.html', data
 
     # IWikiSyntaxProvider methods
 
@@ -1087,22 +1099,22 @@ class MilestoneModule(Component):
                 if hasattr(context, 'req'):
                     if milestone.is_completed:
                         title = _(
-                            'Completed %(duration)s ago (%(date)s)',
+                            "Completed %(duration)s ago (%(date)s)",
                             duration=pretty_timedelta(milestone.completed),
                             date=user_time(context.req, format_datetime,
                                            milestone.completed))
                     elif milestone.is_late:
-                        title = _('%(duration)s late (%(date)s)',
+                        title = _("%(duration)s late (%(date)s)",
                                   duration=pretty_timedelta(milestone.due),
                                   date=user_time(context.req, format_datetime,
                                                  milestone.due))
                     elif milestone.due:
-                        title = _('Due in %(duration)s (%(date)s)',
+                        title = _("Due in %(duration)s (%(date)s)",
                                   duration=pretty_timedelta(milestone.due),
                                   date=user_time(context.req, format_datetime,
                                                  milestone.due))
                     else:
-                        title = _('No date set')
+                        title = _("No date set")
                 closed = 'closed ' if milestone.is_completed else ''
                 return tag.a(label, class_='%smilestone' % closed,
                              href=href + extra, title=title)
@@ -1120,7 +1132,7 @@ class MilestoneModule(Component):
                                  **kwargs):
         desc = resource.id
         if format != 'compact':
-            desc =  _('Milestone %(name)s', name=resource.id)
+            desc = _("Milestone %(name)s", name=resource.id)
         if context:
             return self._render_link(context, resource.id, desc)
         else:
@@ -1146,10 +1158,10 @@ class MilestoneModule(Component):
 
     def get_search_filters(self, req):
         if 'MILESTONE_VIEW' in req.perm:
-            yield ('milestone', _('Milestones'))
+            yield ('milestone', _("Milestones"))
 
     def get_search_results(self, req, terms, filters):
-        if not 'milestone' in filters:
+        if 'milestone' not in filters:
             return
         term_regexps = search_to_regexps(terms)
         milestone_realm = Resource(self.realm)
