@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010 Edgewall Software
+# Copyright (C) 2010-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 try:
     import threading
@@ -25,5 +25,10 @@ class ThreadLocal(threading.local):
         threading.local.__init__(self)
         self.__dict__.update(kwargs)
 
-def get_thread_id():
-    return threading._get_ident()
+
+try:
+    threading.get_ident  # since Python 3.3
+except AttributeError:
+    get_thread_id = threading._get_ident
+else:
+    get_thread_id = threading.get_ident
