@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -22,13 +22,13 @@ from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.config import ConfigSection, Option
 from trac.core import *
 from trac.resource import IResourceManager, Resource, ResourceNotFound
-from trac.util import as_bool
-from trac.util.concurrency import threading
+from trac.util import as_bool, native_path
+from trac.util.concurrency import get_thread_id, threading
 from trac.util.datefmt import time_now, utc
-from trac.util.text import printout, to_unicode, exception_to_unicode
+from trac.util.text import exception_to_unicode, printout, to_unicode
 from trac.util.translation import _
 from trac.web.api import IRequestFilter
-from trac.web.chrome import ITemplateProvider
+from trac.web.chrome import Chrome, ITemplateProvider, add_warning
 
 
 def is_default(reponame):
@@ -47,7 +47,7 @@ class InvalidConnector(TracError):
 class IRepositoryConnector(Interface):
     """Provide support for a specific version control system."""
 
-    error = None # place holder for storing relevant error message
+    error = None  # place holder for storing relevant error message
 
     def get_supported_types():
         """Return the types of version control systems that are supported.
@@ -303,7 +303,7 @@ class DbRepositoryProvider(Component):
                     v = ''
                 if k in ('hidden', 'sync_per_request'):
                     v = '1' if as_bool(v) else None
-                if k == 'dir' and not os.path.isabs(v):
+                if k == 'dir' and not os.path.isabs(native_path(v)):
                     raise TracError(_("The repository directory must be "
                                       "absolute"))
                 db("UPDATE repository SET value=%s WHERE id=%s AND name=%s",
@@ -332,26 +332,25 @@ class RepositoryManager(Component):
     change_listeners = ExtensionPoint(IRepositoryChangeListener)
 
     repositories_section = ConfigSection('repositories',
-        """One of the alternatives for registering new repositories is to
-        populate the `[repositories]` section of the `trac.ini`.
+        """One of the methods for registering repositories is to
+        populate the `[repositories]` section of `trac.ini`.
 
-        This is especially suited for setting up convenience aliases,
-        short-lived repositories, or during the initial phases of an
-        installation.
+        This is especially suited for setting up aliases, using a
+        [TracIni#GlobalConfiguration shared configuration], or specifying
+        repositories at the time of environment creation.
 
-        See [TracRepositoryAdmin#ReposTracIni TracRepositoryAdmin] for details
-        about the format adopted for this section and the rest of that page for
-        the other alternatives.
-
-        (''since 0.12'')""")
+        See [TracRepositoryAdmin#ReposTracIni TracRepositoryAdmin] for
+        details on the format of this section, and look elsewhere on the
+        page for information on other repository providers.
+        """)
 
     default_repository_type = Option('versioncontrol',
                                      'default_repository_type', 'svn',
         """Default repository connector type.
 
-        This is used as the default repository type for repositories defined
-        in the [TracIni#repositories-section repositories] section or using
-        the "Repositories" admin panel. (''since 0.12'')
+        This is used as the default repository type for repositories
+        defined in the [TracIni#repositories-section repositories] section
+        or using the "Repositories" admin panel.
         """)
 
     def __init__(self):
@@ -363,7 +362,6 @@ class RepositoryManager(Component):
     # IRequestFilter methods
 
     def pre_process_request(self, req, handler):
-        from trac.web.chrome import Chrome, add_warning
         if handler is not Chrome(self.env):
             for repo_info in self.get_all_repositories().values():
                 if not as_bool(repo_info.get('sync_per_request')):
@@ -392,15 +390,15 @@ class RepositoryManager(Component):
                         "Failed to sync with repository \"%s\"; You may be "
                         "able to reduce the impact of this issue by "
                         "configuring the sync_per_request option; see "
-                        "http://trac.edgewall.org/wiki/TracRepositoryAdmin"
+                        "https://trac.edgewall.org/wiki/TracRepositoryAdmin"
                         "#ExplicitSync for more detail: %s", repo_name,
                         exception_to_unicode(e, traceback=True))
                 self.log.info("Synchronized '%s' repository in %0.2f seconds",
                               repo_name, time_now() - start)
         return handler
 
-    def post_process_request(self, req, template, data, content_type):
-        return (template, data, content_type)
+    def post_process_request(self, req, template, data, metadata):
+        return template, data, metadata
 
     # IResourceManager methods
 
@@ -461,7 +459,7 @@ class RepositoryManager(Component):
             reponame = resource.id
         else:
             reponame = resource.parent.id
-        repos = self.env.get_repository(reponame)
+        repos = RepositoryManager(self.env).get_repository(reponame)
         if not repos:
             return False
         if resource.realm == self.changeset_realm:
@@ -523,9 +521,10 @@ class RepositoryManager(Component):
 
     def get_supported_types(self):
         """Return the list of supported repository types."""
-        types = set(type_ for connector in self.connectors
-                    for (type_, prio) in connector.get_supported_types() or []
-                    if prio >= 0)
+        types = {type_
+                 for connector in self.connectors
+                 for (type_, prio) in connector.get_supported_types() or []
+                 if prio >= 0}
         return list(types)
 
     def get_repositories_by_dir(self, directory):
@@ -534,10 +533,10 @@ class RepositoryManager(Component):
            :param directory: the key for identifying the repositories.
            :return: list of `Repository` instances.
         """
-        directory = os.path.join(os.path.normcase(directory), '')
+        directory = os.path.join(os.path.normcase(native_path(directory)), '')
         repositories = []
         for reponame, repoinfo in self.get_all_repositories().iteritems():
-            dir = repoinfo.get('dir')
+            dir = native_path(repoinfo.get('dir'))
             if dir:
                 dir = os.path.join(os.path.normcase(dir), '')
                 if dir.startswith(directory):
@@ -584,15 +583,15 @@ class RepositoryManager(Component):
         if 'alias' in repoinfo:
             reponame = repoinfo['alias']
             repoinfo = self.get_all_repositories().get(reponame, {})
-        rdir = repoinfo.get('dir')
+        rdir = native_path(repoinfo.get('dir'))
         if not rdir:
             return None
         rtype = repoinfo.get('type') or self.default_repository_type
 
         # get a Repository for the reponame (use a thread-level cache)
-        with self.env.db_transaction: # prevent possible deadlock, see #4465
+        with self.env.db_transaction:  # prevent possible deadlock, see #4465
             with self._lock:
-                tid = threading._get_ident()
+                tid = get_thread_id()
                 if tid in self._cache:
                     repositories = self._cache[tid]
                 else:
@@ -617,7 +616,7 @@ class RepositoryManager(Component):
         """
         matches = []
         path = path.strip('/') + '/' if path else '/'
-        for reponame in self.get_all_repositories().keys():
+        for reponame in self.get_all_repositories():
             stripped_reponame = reponame.strip('/') + '/'
             if path.startswith(stripped_reponame):
                 matches.append((len(stripped_reponame), reponame))
@@ -650,8 +649,8 @@ class RepositoryManager(Component):
             for provider in self.providers:
                 for reponame, info in provider.get_repositories() or []:
                     if reponame in all_repositories:
-                        self.log.warn("Discarding duplicate repository '%s'",
-                                      reponame)
+                        self.log.warning("Discarding duplicate repository "
+                                         "'%s'", reponame)
                     else:
                         info['name'] = reponame
                         if 'id' not in info:
@@ -661,16 +660,19 @@ class RepositoryManager(Component):
         return self._all_repositories
 
     def get_real_repositories(self):
-        """Return a set of all real repositories (i.e. excluding aliases)."""
+        """Return a sorted list of all real repositories (i.e. excluding
+        aliases).
+        """
         repositories = set()
         for reponame in self.get_all_repositories():
             try:
                 repos = self.get_repository(reponame)
+            except TracError:
+                pass  # Skip invalid repositories
+            else:
                 if repos is not None:
                     repositories.add(repos)
-            except TracError:
-                pass # Skip invalid repositories
-        return repositories
+        return sorted(repositories, key=lambda r: r.reponame)
 
     def reload_repositories(self):
         """Reload the repositories from the providers."""
@@ -706,8 +708,8 @@ class RepositoryManager(Component):
             repositories = [r for r in self.get_real_repositories()
                             if r.get_base() == base]
         if not repositories:
-            self.log.warn("Found no repositories matching '%s' base.",
-                          base or reponame)
+            self.log.warning("Found no repositories matching '%s' base.",
+                             base or reponame)
             return [_("Repository '%(repo)s' not found",
                       repo=reponame or _("(default)"))]
 
@@ -722,7 +724,7 @@ class RepositoryManager(Component):
                         old_changeset = repos.sync_changeset(rev)
                     except NoSuchChangeset as e:
                         errors.append(exception_to_unicode(e))
-                        self.log.warn(
+                        self.log.warning(
                             "No changeset '%s' found in repository '%s'. "
                             "Skipping subscribers for event %s",
                             rev, reponame, event)
@@ -737,7 +739,7 @@ class RepositoryManager(Component):
                         changeset = repos.get_changeset(rev)
                     except NoSuchChangeset as e:
                         errors.append(exception_to_unicode(e))
-                        self.log.warn(
+                        self.log.warning(
                             "No changeset '%s' found in repository '%s'. "
                             "Skipping subscribers for event %s",
                             rev, reponame, event)
@@ -751,7 +753,7 @@ class RepositoryManager(Component):
     def shutdown(self, tid=None):
         """Free `Repository` instances bound to a given thread identifier"""
         if tid:
-            assert tid == threading._get_ident()
+            assert tid == get_thread_id()
             with self._lock:
                 repositories = self._cache.pop(tid, {})
                 for reponame, repos in repositories.iteritems():
@@ -802,7 +804,7 @@ class RepositoryManager(Component):
                         self._connectors[type_] = keep
         if rtype in self._connectors:
             connector, prio = self._connectors[rtype]
-            if prio >= 0: # no error condition
+            if prio >= 0:  # no error condition
                 return connector
             else:
                 raise InvalidConnector(
@@ -1027,20 +1029,15 @@ class Repository(object):
         """
         pass
 
-    # @abstractmethod
+    @abstractmethod
     def get_path_history(self, path, rev=None, limit=None):
         """Retrieve all the revisions containing this path.
 
         If given, `rev` is used as a starting point (i.e. no revision
         ''newer'' than `rev` should be returned).
         The result format should be the same as the one of Node.get_history()
-
-        :since 1.1.2: The method should be implemented in subclasses since
-                      it will be made abstract in Trac 1.3.1. A `TypeError`
-                      will result when instantiating classes that don't
-                      implement the method.
         """
-        raise NotImplementedError
+        pass
 
     @abstractmethod
     def normalize_path(self, path):
@@ -1272,7 +1269,7 @@ class Changeset(object):
     MOVE = 'move'
 
     # change types which can have diff associated to them
-    DIFF_CHANGES = (EDIT, COPY, MOVE) # MERGE
+    DIFF_CHANGES = (EDIT, COPY, MOVE)  # MERGE
     OTHER_CHANGES = (ADD, DELETE)
     ALL_CHANGES = DIFF_CHANGES + OTHER_CHANGES
 
