@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -29,7 +29,7 @@ _actionmap = {'A': Changeset.ADD, 'C': Changeset.COPY,
               'M': Changeset.MOVE}
 
 def _invert_dict(d):
-    return dict(zip(d.values(), d.keys()))
+    return dict(zip(d.values(), list(d)))
 
 _inverted_kindmap = _invert_dict(_kindmap)
 _inverted_actionmap = _invert_dict(_actionmap)
@@ -282,36 +282,30 @@ class CachedRepository(Repository):
     def insert_changeset(self, rev, cset):
         """Create revision and node_change records for the given changeset
         instance."""
-        with self.env.db_transaction as db:
-            self._insert_changeset(db, rev, cset)
-
-    def _insert_changeset(self, db, rev, cset):
-        """:deprecated: since 1.1.2, use `insert_changeset` instead. Will
-                        be removed in 1.3.1.
-        """
         srev = self.db_rev(rev)
-        # 1. Attempt to resync the 'revision' table.  In case of
-        # concurrent syncs, only such insert into the `revision` table
-        # will succeed, the others will fail and raise an exception.
-        db("""
-            INSERT INTO revision (repos,rev,time,author,message)
-            VALUES (%s,%s,%s,%s,%s)
-            """, (self.id, srev, to_utimestamp(cset.date),
-                  cset.author, cset.message))
-        # 2. now *only* one process was able to get there (i.e. there
-        # *shouldn't* be any race condition here)
-        for path, kind, action, bpath, brev in cset.get_changes():
-            self.log.debug("Caching node change in [%s] in '%s': %r",
-                           rev, _norm_reponame(self.repos),
-                           (path, kind, action, bpath, brev))
-            kind = _inverted_kindmap[kind]
-            action = _inverted_actionmap[action]
+        with self.env.db_transaction as db:
+            # 1. Attempt to resync the 'revision' table.  In case of
+            # concurrent syncs, only such insert into the `revision` table
+            # will succeed, the others will fail and raise an exception.
             db("""
-                INSERT INTO node_change
-                    (repos,rev,path,node_type,change_type,base_path,
-                     base_rev)
-                VALUES (%s,%s,%s,%s,%s,%s,%s)
-                """, (self.id, srev, path, kind, action, bpath, brev))
+                INSERT INTO revision (repos,rev,time,author,message)
+                VALUES (%s,%s,%s,%s,%s)
+                """, (self.id, srev, to_utimestamp(cset.date),
+                      cset.author, cset.message))
+            # 2. now *only* one process was able to get there (i.e. there
+            # *shouldn't* be any race condition here)
+            for path, kind, action, bpath, brev in cset.get_changes():
+                self.log.debug("Caching node change in [%s] in '%s': %r",
+                               rev, _norm_reponame(self.repos),
+                               (path, kind, action, bpath, brev))
+                kind = _inverted_kindmap[kind]
+                action = _inverted_actionmap[action]
+                db("""
+                    INSERT INTO node_change
+                        (repos,rev,path,node_type,change_type,base_path,
+                         base_rev)
+                    VALUES (%s,%s,%s,%s,%s,%s,%s)
+                    """, (self.id, srev, path, kind, action, bpath, brev))
 
     def get_node(self, path, rev=None):
         return self.repos.get_node(path, self.normalize_rev(rev))
@@ -347,9 +341,8 @@ class CachedRepository(Repository):
                                                         in node_infos]
         sfirst = self.db_rev(min(first for node, first in node_infos))
         slast = self.db_rev(max(node.rev for node, first in node_infos))
-        path_infos = dict((node.path, (node, first)) for node, first
-                                                     in node_infos)
-        path_revs = dict((node.path, []) for node, first in node_infos)
+        path_infos = {node.path: (node, first) for node, first in node_infos}
+        path_revs = {node.path: [] for node, first in node_infos}
 
         # Prevent "too many SQL variables" since max number of parameters is
         # 999 on SQLite. No limitation on PostgreSQL and MySQL.
@@ -440,7 +433,7 @@ class CachedRepository(Repository):
                 components = path.lstrip('/').split('/')
                 parents = ','.join(('%s',) * len(components))
                 sql += " OR (path IN (" + parents + ") AND change_type='D'))"
-                for i in range(1, len(components) + 1):
+                for i in xrange(1, len(components) + 1):
                     args.append('/'.join(components[:i]))
             else:
                 sql %= {'aggr': aggr, 'dir': direction, 'tab': 'revision'}
