@@ -1,25 +1,23 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2009 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 import difflib
 import re
 
-from genshi import Markup, escape
-
-from trac.util import as_int
+from trac.util.html import Markup, escape
 from trac.util.text import expandtabs
 
 __all__ = ['diff_blocks', 'get_change_extent', 'get_diff_options',
@@ -49,7 +47,7 @@ def get_change_extent(str1, str2):
     limit = limit - start
     while -end <= limit and str1[end] == str2[end]:
         end -= 1
-    return (start, end + 1)
+    return start, end + 1
 
 
 def get_filtered_hunks(fromlines, tolines, context=None,
@@ -121,7 +119,7 @@ def filter_ignorable_lines(hunks, fromlines, tolines, context,
                 if ignore_space_changes:
                     str = _norm_space_changes(str)
                 return str
-            for i in range(len(fromlines)):
+            for i in xrange(len(fromlines)):
                 if f(fromlines[i]) != f(tolines[i]):
                     return False
             return True
@@ -187,10 +185,6 @@ def filter_ignorable_lines(hunks, fromlines, tolines, context,
             yield hunk
 
 
-def hdf_diff(*args, **kwargs):
-    """:deprecated: use `diff_blocks` (will be removed in 1.1.1)"""
-    return diff_blocks(*args, **kwargs)
-
 def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
                 ignore_blank_lines=0, ignore_case=0, ignore_space_changes=0):
     """Return an array that is adequate for adding to the data dictionary
@@ -206,12 +200,12 @@ def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
     space_re = re.compile(' ( +)|^ ')
     def htmlify(match):
         div, mod = divmod(len(match.group(0)), 2)
-        return div * '&nbsp; ' + mod * '&nbsp;'
+        return Markup(div * '&nbsp; ' + mod * '&nbsp;')
 
     def markup_intraline_changes(opcodes):
         for tag, i1, i2, j1, j2 in opcodes:
             if tag == 'replace' and i2 - i1 == j2 - j1:
-                for i in range(i2 - i1):
+                for i in xrange(i2 - i1):
                     fromline, toline = fromlines[i1 + i], tolines[j1 + i]
                     (start, end) = get_change_extent(fromline, toline)
                     if start != 0 or end != 0:
@@ -250,8 +244,8 @@ def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
                     for line in fromlines[i1:i2]:
                         line = expandtabs(line, tabwidth, '\0\1')
                         line = escape(line, quotes=False)
-                        line = '<del>'.join([space_re.sub(htmlify, seg)
-                                             for seg in line.split('\0')])
+                        line = '<del>'.join(space_re.sub(htmlify, seg)
+                                            for seg in line.split('\0'))
                         line = line.replace('\1', '</del>')
                         blocks[-1]['base']['lines'].append(
                             Markup(unicode(line)))
@@ -259,8 +253,8 @@ def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
                     for line in tolines[j1:j2]:
                         line = expandtabs(line, tabwidth, '\0\1')
                         line = escape(line, quotes=False)
-                        line = '<ins>'.join([space_re.sub(htmlify, seg)
-                                             for seg in line.split('\0')])
+                        line = '<ins>'.join(space_re.sub(htmlify, seg)
+                                            for seg in line.split('\0'))
                         line = line.replace('\1', '</ins>')
                         blocks[-1]['changed']['lines'].append(
                             Markup(unicode(line)))
@@ -317,7 +311,7 @@ def get_diff_options(req):
     data = {'options': options_data}
 
     def get_bool_option(name, default=0):
-        pref = int(req.session.get('diff_' + name, default))
+        pref = req.session.as_int('diff_' + name, default)
         arg = int(name in req.args)
         if 'update' in req.args and arg != pref:
             req.session.set('diff_' + name, arg, default)
@@ -331,13 +325,13 @@ def get_diff_options(req):
         req.session.set('diff_style', style, 'inline')
     data['style'] = style
 
-    pref = int(req.session.get('diff_contextlines', 2))
-    context = as_int(req.args.get('contextlines', pref), -1)
+    pref = req.session.as_int('diff_contextlines', 2)
+    context = req.args.as_int('contextlines', pref)
     if 'update' in req.args and context != pref:
         req.session.set('diff_contextlines', context, 2)
     options_data['contextlines'] = context
 
-    arg = as_int(req.args.get('contextall'), 0)
+    arg = req.args.as_int('contextall', 0)
     options_data['contextall'] = arg
     options = ['-U%d' % (-1 if arg else context)]
 
@@ -356,4 +350,4 @@ def get_diff_options(req):
         options.append('-b')
     options_data['ignorewhitespace'] = arg
 
-    return (style, options, data)
+    return style, options, data
