@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
@@ -19,8 +19,6 @@ import re
 from datetime import datetime, timedelta
 from fnmatch import fnmatchcase
 
-from genshi.builder import tag
-
 from trac.config import BoolOption, ListOption, Option
 from trac.core import *
 from trac.mimeview.api import IHTMLPreviewAnnotator, Mimeview, is_binary
@@ -28,15 +26,15 @@ from trac.perm import IPermissionRequestor, PermissionError
 from trac.resource import Resource, ResourceNotFound
 from trac.util import as_bool, embedded_numbers
 from trac.util.datefmt import datetime_now, http_date, to_datetime, utc
-from trac.util.html import Markup, escape
+from trac.util.html import Markup, escape, tag
 from trac.util.text import exception_to_unicode, shorten_line
 from trac.util.translation import _, cleandoc_
 from trac.versioncontrol.api import NoSuchChangeset, RepositoryManager
 from trac.versioncontrol.web_ui.util import *
 from trac.web.api import IRequestHandler, RequestDone
-from trac.web.chrome import (INavigationContributor, add_ctxtnav, add_link,
-                             add_script, add_stylesheet, prevnext_nav,
-                             web_context)
+from trac.web.chrome import (Chrome, INavigationContributor, add_ctxtnav,
+                             add_link, add_script, add_stylesheet,
+                             prevnext_nav, web_context)
 from trac.wiki.api import IWikiMacroProvider, IWikiSyntaxProvider, parse_args
 from trac.wiki.formatter import format_to_html, format_to_oneliner
 
@@ -78,7 +76,7 @@ class IPropertyRenderer(Interface):
          - a `RenderedProperty` instance: the property will only be displayed
            using the instance's `content` attribute, and the other attributes
            will also be used in some display contexts (like `revprop`)
-         - `Markup` or other Genshi content: the property will be displayed
+         - `Markup` or `Fragment`: the property will be displayed
            normally, using that content as a block-level markup
         """
 
@@ -104,8 +102,8 @@ class DefaultPropertyRenderer(Component):
         # No special treatment besides respecting newlines in values.
         value = props[name]
         if value and '\n' in value:
-            value = Markup(''.join(['<br />%s' % escape(v)
-                                    for v in value.split('\n')]))
+            value = Markup(''.join('<br />%s' % escape(v)
+                                   for v in value.split('\n')))
         return value
 
 
@@ -332,7 +330,7 @@ class BrowserModule(Component):
             req.redirect(presel)
 
         path = req.args.get('path', '/')
-        rev = req.args.getfirst('rev', '')
+        rev = req.args.get('rev', '')
         if rev.lower() in ('', 'head'):
             rev = None
         format = req.args.get('format')
@@ -433,12 +431,11 @@ class BrowserModule(Component):
             'order': order, 'desc': 1 if desc else None,
             'repo': repo_data, 'dir': dir_data, 'file': file_data,
             'quickjump_entries': quickjump_data,
-            'wiki_format_messages': \
+            'wiki_format_messages':
                 self.config['changeset'].getbool('wiki_format_messages'),
-            'xhr': req.is_xhr,  # Remove in 1.3.1
         }
         if req.is_xhr: # render and return the content only
-            return 'dir_entries.html', data, None
+            return 'dir_entries.html', data
 
         if dir_data or repo_data:
             add_script(req, 'common/js/expand_dir.js')
@@ -499,7 +496,7 @@ class BrowserModule(Component):
                 add_ctxtnav(req, _('Repository URL'), href=path_url)
 
         add_stylesheet(req, 'common/css/browser.css')
-        return 'browser.html', data, None
+        return 'browser.html', data
 
     # Internal methods
 
@@ -546,15 +543,18 @@ class BrowserModule(Component):
 
         # Ordering of repositories
         if order == 'date':
-            def repo_order((reponame, repoinfo, repos, youngest, err, href)):
+            def repo_order(args):
+                reponame, repoinfo, repos, youngest, err, href = args
                 return (youngest.date if youngest else to_datetime(0),
                         embedded_numbers(reponame.lower()))
         elif order == 'author':
-            def repo_order((reponame, repoinfo, repos, youngest, err, href)):
+            def repo_order(args):
+                reponame, repoinfo, repos, youngest, err, href = args
                 return (youngest.author.lower() if youngest else '',
                         embedded_numbers(reponame.lower()))
         else:
-            def repo_order((reponame, repoinfo, repos, youngest, err, href)):
+            def repo_order(args):
+                reponame, repoinfo, repos, youngest, err, href = args
                 return embedded_numbers(reponame.lower())
 
         repositories = sorted(repositories, key=repo_order, reverse=desc)
@@ -593,7 +593,7 @@ class BrowserModule(Component):
             timerange = TimeRange(newest)
             max_s = req.args.get('range_max_secs')
             min_s = req.args.get('range_min_secs')
-            parent_range = [timerange.from_seconds(long(s))
+            parent_range = [timerange.from_seconds(int(s))
                             for s in [max_s, min_s] if s]
             this_range = [c.date for c in changes.values() if c]
             for dt in this_range + parent_range:
@@ -862,7 +862,7 @@ class BrowserModule(Component):
                         else _("Download as Zip archive")
         except TracError:
             pass
-        return (node, raw_href, title)
+        return node, raw_href, title
 
     # IHTMLPreviewAnnotator methods
 
@@ -901,8 +901,6 @@ class BrowserModule(Component):
             "author")
           ''desc''::
             When set to 1, order by descending order
-
-        (''since 0.12'')
         """)
         return 'messages', description
 
@@ -927,10 +925,9 @@ class BrowserModule(Component):
             data = {'repo': repo, 'order': order, 'desc': 1 if desc else None,
                     'reponame': None, 'path': '/', 'stickyrev': None,
                     'wiki_format_messages': wiki_format_messages}
-            from trac.web.chrome import Chrome
-            return Chrome(self.env).render_template(
-                    formatter.req, 'repository_index.html', data, None,
-                    fragment=True)
+            return Chrome(self.env).render_fragment(formatter.context.req,
+                                                    'repository_index.html',
+                                                    data)
 
         def get_repository(reponame):
             try:
@@ -958,8 +955,8 @@ class BrowserModule(Component):
                     tag.dd(repos.params.get('description')))
                 for reponame, repos in all_repos])
         else: # compact
-            return Markup(', ').join([repolink(reponame, repos)
-                                      for reponame, repos in all_repos])
+            return Markup(', ').join(repolink(reponame, repos)
+                                     for reponame, repos in all_repos)
 
 
 
@@ -994,7 +991,7 @@ class BlameAnnotator(object):
         chgset = self.repos.get_changeset(rev)
         chgsets = {rev: chgset}
         self.timerange = TimeRange(chgset.date)
-        for idx in range(len(self.annotations)):
+        for idx in xrange(len(self.annotations)):
             rev = self.annotations[idx]
             chgset = chgsets.get(rev)
             if not chgset:
