@@ -1,41 +1,39 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christian Boos <cboos@edgewall.org>
 
 import re
 
-from genshi.core import Markup
-from genshi.builder import tag
-
 from trac.config import IntOption, ListOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.resource import ResourceNotFound
-from trac.util import Ranges, as_int
+from trac.util import Ranges
+from trac.util.html import Markup, tag
 from trac.util.text import to_unicode, wrap
 from trac.util.translation import _
 from trac.versioncontrol.api import (Changeset, NoSuchChangeset,
                                      RepositoryManager)
 from trac.versioncontrol.web_ui.changeset import ChangesetModule
 from trac.versioncontrol.web_ui.util import *
-from trac.web import IRequestHandler
-from trac.web.chrome import (Chrome, INavigationContributor, add_ctxtnav,
-                             add_link, add_script, add_script_data,
-                             add_stylesheet, auth_link, web_context)
+from trac.web.api import IRequestHandler
+from trac.web.chrome import (INavigationContributor, add_ctxtnav, add_link,
+                             add_script, add_script_data, add_stylesheet,
+                             auth_link, web_context)
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
 
@@ -44,9 +42,11 @@ class LogModule(Component):
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
                IWikiSyntaxProvider)
 
+    realm = RepositoryManager.changeset_realm
+
     default_log_limit = IntOption('revisionlog', 'default_log_limit', 100,
         """Default value for the limit argument in the TracRevisionLog.
-        (''since 0.11'')""")
+        """)
 
     graph_colors = ListOption('revisionlog', 'graph_colors',
         ['#cc0', '#0c0', '#0cc', '#00c', '#c0c', '#c00'],
@@ -84,7 +84,7 @@ class LogModule(Component):
         revs = req.args.get('revs')
         format = req.args.get('format')
         verbose = req.args.get('verbose')
-        limit = as_int(req.args.get('limit'), self.default_log_limit)
+        limit = req.args.getint('limit', self.default_log_limit)
 
         rm = RepositoryManager(self.env)
         reponame, repos, path = rm.get_repository_by_path(path)
@@ -120,7 +120,7 @@ class LogModule(Component):
         #    unless explicit ranges have been specified
         #  * for ''show only add, delete'' we're using
         #   `Repository.get_path_history()`
-        cset_resource = repos.resource.child('changeset')
+        cset_resource = repos.resource.child(self.realm)
         show_graph = False
         curr_revrange = []
         if mode == 'path_history':
@@ -288,13 +288,13 @@ class LogModule(Component):
         }
 
         if format == 'changelog':
-            return 'revisionlog.txt', data, 'text/plain'
+            return 'revisionlog.txt', data, {'content_type': 'text/plain'}
         elif format == 'rss':
-            data['email_map'] = Chrome(self.env).get_email_map()
             data['context'] = web_context(req, 'source',
                                           path, parent=repos.resource,
                                           absurls=True)
-            return 'revisionlog.rss', data, 'application/rss+xml'
+            return ('revisionlog.rss', data,
+                    {'content_type': 'application/rss+xml'})
 
         item_ranges = []
         range = []
@@ -336,7 +336,7 @@ class LogModule(Component):
                                             href=next['href']),
                                       Markup(' &rarr;')))
 
-        return 'revisionlog.html', data, None
+        return 'revisionlog.html', data
 
     # IWikiSyntaxProvider methods
 
@@ -418,7 +418,7 @@ class LogModule(Component):
                 errmsg = _("Repository '%(repo)s' not found", repo=reponame)
             else:
                 errmsg = _("No default repository defined")
-        except TracError, e:
+        except TracError as e:
             errmsg = to_unicode(e)
         return tag.a(label, class_='missing source', title=errmsg)
 
