@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Jonas Borgström <jonas@edgewall.com>
@@ -9,11 +9,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -24,7 +24,7 @@ from trac.admin.api import AdminCommandError, IAdminCommandProvider, \
                            console_date_format, get_console_locale
 from trac.core import Component, ExtensionPoint, TracError, TracValueError, \
                       implements
-from trac.util import as_bool, as_int, hex_entropy, lazy
+from trac.util import as_bool, as_float, as_int, hex_entropy, lazy
 from trac.util.datefmt import get_datetime_format_hint, format_date, \
                               parse_date, time_now, to_datetime, to_timestamp
 from trac.util.text import print_table
@@ -75,8 +75,8 @@ class SessionDict(dict):
         integer.
 
         :param key: the name of the session attribute
-        :keyword default: the value to return if the parameter is not
-                          specified or an exception occurs converting
+        :keyword default: the value to return if the parameter does
+                          not exist or an exception occurs converting
                           the value to an integer.
         :keyword min: lower bound to which the value is limited
         :keyword max: upper bound to which the value is limited
@@ -87,6 +87,24 @@ class SessionDict(dict):
             return default
         return as_int(self[key], default, min, max)
 
+    def as_float(self, key, default=None, min=None, max=None):
+        """Return the value as a float. Return `default` if
+        if an exception is raised while converting the value to a
+        float.
+
+        :param key: the name of the session attribute
+        :keyword default: the value to return if the parameter does
+                          not exist or an exception occurs converting
+                          the value to a float.
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.3.6
+        """
+        if key not in self:
+            return default
+        return as_float(self[key], default, min, max)
+
     def set(self, key, value, default=None):
         """Set a variable in the session, or remove it if it's equal to the
         default value.
@@ -245,7 +263,7 @@ class Session(DetachedSession):
     def __init__(self, env, req):
         super(Session, self).__init__(env, None)
         self.req = req
-        if req.authname == 'anonymous':
+        if not req.is_authenticated:
             if COOKIE_KEY not in req.incookie:
                 self.sid = hex_entropy(24)
                 self.bake_cookie()
@@ -286,7 +304,7 @@ class Session(DetachedSession):
             self.bake_cookie()
 
     def change_sid(self, new_sid):
-        assert self.req.authname == 'anonymous', \
+        assert not self.req.is_authenticated, \
                'Cannot change ID of authenticated session'
         assert new_sid, 'Session ID cannot be empty'
         if new_sid == self.sid:
@@ -314,7 +332,7 @@ class Session(DetachedSession):
         """Promotes an anonymous session to an authenticated session, if there
         is no preexisting session data for that user name.
         """
-        assert self.req.authname != 'anonymous', \
+        assert self.req.is_authenticated, \
                "Cannot promote session of anonymous user"
 
         with self.env.db_transaction as db:
@@ -414,7 +432,7 @@ class SessionAdmin(Component):
                self._complete_delete, self._do_delete)
 
         yield ('session purge', '<age>',
-               """Purge anonymous sessions older than the given age or date
+               """Purge anonymous sessions older than given age or date
 
                Age may be specified as a relative time like "90 days ago", or
                as a date in the "%(datetime)s" or "%(iso8601)s" (ISO 8601)
@@ -442,8 +460,9 @@ class SessionAdmin(Component):
     def _get_list(self, sids):
         all_anon = 'anonymous' in sids or '*' in sids
         all_auth = 'authenticated' in sids or '*' in sids
-        sids = set(self._split_sid(sid) for sid in sids
-                   if sid not in ('anonymous', 'authenticated', '*'))
+        sids = {self._split_sid(sid)
+                for sid in sids
+                if sid not in ('anonymous', 'authenticated', '*')}
         rows = self.env.db_query("""
             SELECT DISTINCT s.sid, s.authenticated, s.last_visit,
                             n.value, e.value, h.value
