@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import difflib
+import io
 import os
 import re
 import unittest
-import StringIO
 
 # Python 2.7 `assertMultiLineEqual` calls `safe_repr(..., short=True)`
 # which breaks our custom failure display in WikiTestCase.
@@ -39,14 +39,16 @@ class WikiTestCase(unittest.TestCase):
 
     generate_opts = {}
 
-    def __init__(self, title, input, correct, file, line, setup=None,
-                 teardown=None, context=None, default_data=False,
+    def __init__(self, title, input, expected, file, line,
+                 setup=None, teardown=None, context=None, default_data=False,
                  enable_components=None, disable_components=None,
                  env_path='', destroying=False):
         unittest.TestCase.__init__(self, 'test')
         self.title = title
         self.input = input
-        self.correct = correct
+        self.expected = expected
+        if file.endswith('.pyc'):
+            file = file.replace('.pyc', '.py')
         self.file = file
         self.line = line
         self._setup = setup
@@ -61,8 +63,8 @@ class WikiTestCase(unittest.TestCase):
     def _create_env(self):
         env = EnvironmentStub(**self._env_kwargs)
         # -- intertrac support
-        env.config.set('intertrac', 'trac.title', "Trac's Trac")
-        env.config.set('intertrac', 'trac.url', "http://trac.edgewall.org")
+        env.config.set('intertrac', 'genshi.title', "Genshi's Trac")
+        env.config.set('intertrac', 'genshi.url', "https://genshi.edgewall.org")
         env.config.set('intertrac', 't', 'trac')
         env.config.set('intertrac', 'th.title', "Trac Hacks")
         env.config.set('intertrac', 'th.url', "http://trac-hacks.org")
@@ -88,9 +90,9 @@ class WikiTestCase(unittest.TestCase):
         self.env.href = self.req.href
         self.env.abs_href = self.req.abs_href
         self.env.db_transaction(
-            "INSERT INTO wiki VALUES(%s,%s,%s,%s,%s,%s,%s,%s)",
+            "INSERT INTO wiki VALUES(%s,%s,%s,%s,%s,%s,%s)",
             ('WikiStart', 1, to_utimestamp(datetime_now(utc)), 'joe',
-             '::1', '--', 'Entry page', 0))
+             '--', 'Entry page', 0))
         if self._setup:
             self._setup(self)
 
@@ -106,7 +108,7 @@ class WikiTestCase(unittest.TestCase):
         v = v.replace('\r', '').replace(u'\u200b', '')  # FIXME: keep ZWSP
         v = strip_line_ws(v, leading=False)
         try:
-            self.assertEqual(self.correct, v)
+            self.assertEqual(self.expected, v)
         except AssertionError as e:
             msg = to_unicode(e)
             match = re.match(r"u?'(.*)' != u?'(.*)'", msg)
@@ -159,7 +161,7 @@ class OutlineTestCase(WikiTestCase):
                 self.outliner = OutlineFormatter(env, context)
                 self.input = input
             def generate(self):
-                out = StringIO.StringIO()
+                out = io.StringIO()
                 self.outliner.format(self.input, out)
                 return out.getvalue()
         return Outliner(self.env, self.context, self.input)
@@ -188,16 +190,16 @@ def wikisyntax_test_suite(data=None, setup=None, file=None, teardown=None,
             if len(blocks) < 5:
                 blocks.extend([None] * (5 - len(blocks)))
             input, page, oneliner, page_escape_nl, outline = blocks[:5]
-            for cls, correct in [
+            for cls, expected in [
                     (WikiTestCase, page),
                     (OneLinerTestCase, oneliner and oneliner[:-1]),
                     (EscapeNewLinesTestCase, page_escape_nl),
                     (OutlineTestCase, outline)]:
-                if correct:
-                    tc = cls(title, input, correct, filename, line, setup,
-                             teardown, context, default_data,
-                             enable_components, disable_components, env_path,
-                             destroying)
+                if expected:
+                    tc = cls(title, input, expected, filename, line,
+                             setup, teardown, context, default_data,
+                             enable_components, disable_components,
+                             env_path, destroying)
                     suite.addTest(tc)
 
     if data:
