@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2018 Edgewall Software
+# Copyright (C) 2009-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from trac.core import *
 from trac.config import ConfigSection
@@ -32,7 +32,7 @@ class ExtraPermissionsProvider(Component):
         To add new permissions, create a new section `[extra-permissions]` in
         your `trac.ini`. Every entry in that section defines a meta-permission
         and a comma-separated list of permissions. For example:
-        {{{
+        {{{#!ini
         [extra-permissions]
         EXTRA_ADMIN = EXTRA_VIEW, EXTRA_MODIFY, EXTRA_DELETE
         }}}
@@ -46,7 +46,7 @@ class ExtraPermissionsProvider(Component):
 
         If you don't want a meta-permission, start the meta-name with an
         underscore (`_`):
-        {{{
+        {{{#!ini
         [extra-permissions]
         _perms = EXTRA_VIEW, EXTRA_MODIFY
         }}}
