@@ -1,31 +1,30 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Christian Boos <cboos@edgewall.org>
 
 import posixpath
 
-from genshi.builder import tag
-
 from trac.config import ConfigSection
 from trac.core import *
 from trac.versioncontrol.api import NoSuchNode, RepositoryManager
 from trac.versioncontrol.web_ui.browser import IPropertyRenderer
 from trac.versioncontrol.web_ui.changeset import IPropertyDiffRenderer
 from trac.util import Ranges, to_ranges
+from trac.util.html import tag
 from trac.util.translation import _, tag_
 from trac.web.chrome import chrome_resource_path
 from tracopt.versioncontrol.svn.svn_fs import _path_within_scope
@@ -59,8 +58,7 @@ class SubversionPropertyRenderer(Component):
         be mapped to `http://ourserver/tracs/tools/browser/tags/1.1/tools?rev=`
         (and `rev` will be set to the appropriate revision number if the
         external additionally specifies a revision, see the
-        [http://svnbook.red-bean.com/en/1.4/svn.advanced.externals.html SVN Book on externals]
-        for more details).
+        [%(svnbook)s SVN Book on externals] for more details).
 
         Note that the number used as a key in the above section is purely used
         as a place holder, as the URLs themselves can't be used as a key due to
@@ -69,8 +67,8 @@ class SubversionPropertyRenderer(Component):
         Finally, the relative URLs introduced in
         [http://subversion.apache.org/docs/release-notes/1.5.html#externals Subversion 1.5]
         are not yet supported.
-
-        (''since 0.11'')""")
+        """,
+        doc_args={'svnbook': 'http://svnbook.red-bean.com/en/1.7/svn.advanced.externals.html'})
 
     def __init__(self):
         self._externals_map = {}
@@ -99,9 +97,9 @@ class SubversionPropertyRenderer(Component):
             for dummykey, value in self.svn_externals_section.options():
                 value = value.split()
                 if len(value) != 2:
-                    self.log.warn("svn:externals entry %s doesn't contain "
-                                  "a space-separated key value pair, "
-                                  "skipping.", dummykey)
+                    self.log.warning("svn:externals entry %s doesn't contain "
+                                     "a space-separated key value pair, "
+                                     "skipping.", dummykey)
                     continue
                 key, value = value
                 self._externals_map[key] = value.replace('%', '%%') \
@@ -273,10 +271,12 @@ class SubversionMergePropertyRenderer(Component):
         if not rows:
             return None
         rows.sort()
-        has_deleted = rows[-1][0] if rows else None
-        return tag(has_deleted and tag.a(_('(toggle deleted branches)'),
-                                         class_='trac-toggledeleted',
-                                         href='#'),
+        if rows and rows[-1][0]:
+            toggledeleted = tag.a(_("(toggle deleted branches)"),
+                                  class_='trac-toggledeleted', href='#')
+        else:
+            toggledeleted = None
+        return tag(toggledeleted,
                    tag.table(tag.tbody(
                        [tag.tr(row, class_='trac-deleted' if deleted else None)
                         for deleted, spath, row in rows]), class_='props'))
@@ -354,7 +354,10 @@ class SubversionMergePropertyDiffRenderer(Component):
         repos = rm.get_repository(old_context.resource.parent.id)
         def parse_sources(props):
             sources = {}
-            for line in props[name].splitlines():
+            value = props[name]
+            lines = value.splitlines() if name == 'svn:mergeinfo' \
+                                       else value.split()
+            for line in lines:
                 path, revs = line.split(':', 1)
                 spath = _path_within_scope(repos.scope, path)
                 if spath is not None:
